/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview The file, tooltip.js, tracks the position of the mouse pointer,
 * and when placed over a designated element that contains more information,
 * creates a tool tip that follows the movement of the mouse as long as the pointer
 * does not leave the designated element. To make this a more functional tool tip,
 * the code keeps track of the browser's size (even through a window resize) and
 * positions the tool tip around the designated element based on where it will fit
 * within the browser without spilling over. This makes sure that the tool tip
 * will never be cut off or create unnecessary scroll bars.
 *
 * Elements that are designated as having more information to be shown in a tool
 * tip have a class value of "toolTip" with a complementing element for the tool
 * tip with an id value of the designated element's id value + "Def". An example
 * would be an element:
 *
 * <span id="tip1" class="toolTip">word</span>
 *
 * with a complementing element for the tool tip:
 *
 * <div id="tip1Def" class="toolTipDef">Word tool-tip</div>
 *
 * The designated element can be any type of element, really, but the element that
 * will be the tool tip must be a block-level element, with the easiest one to
 * choose being the <div> element.
 */

/**
 * This variable keeps track of the mouse position for all tool tips.
 */
var mousePosition = null;
/**
 * This variable keeps track of the window's size for all tool tips.
 */var windowSize = null;

/**
 * This function, mouseMovement, takes an event /e/ as its parameter and sets the
 * global /mousePosition/ variable to an object containing the [x, y] pair
 * representing the position of the mouse pointer when the event happened.
 *
 * @param {Event} e The event that has fired in the browser.
 */
function mouseMovement(e) {
	e = e || window.event;
	/* Is this Internet Explorer? */
	if (e.pageX || e.pageY)
		mousePosition = {
			x: e.pageX,
			y: e.pageY
		};
	else
		mousePosition = {
			x: e.clientX + document.body.scrollLeft - document.body.clientLeft,
			y: e.clientY + document.body.scrollTop - document.body.clientTop
		};
}

/**
 * This function, getResizedWindow, sets the global /windowSize/ variable to an
 * object containing the [x, y] pair representing the width and height of the
 * browser window.
 */
function getResizedWindow( ) {
	windowSize = {
		x: ((document.body.clientWidth) ? document.body.clientWidth : window.innerWidth),
		y: ((document.body.clientHeight) ? document.body.clientHeight : window.innerHeight)
	};
}

try {
	/* Set up the global events to keep track of mouse movement and window size */
	Event.observe(document, 'mousemove', mouseMovement, false);
	Event.observe(window, 'resize', getResizedWindow, false);
} catch (ex) {}

/**
 * This function, positionToolTip, takes the passed /p_tip/ and calculates where on
 * the page the tool tip should appear on the screen. This function takes into
 * consideration the outside edges of the browser window to make sure nothing is cut
 * off or causes unnecessary scrolling. It returns an object with the top and left
 * positions for the tool tip.
 *
 * @param {Node} p_tip The node that is the active tool tip.
 * @return An object containing the top and left positions for the tool tip.
 * @type Object
 */
function positionToolTip(p_tip) {
	/* Calculate the top and left corners of the tool tip */
	var tipTop = mousePosition.y - p_tip.clientHeight - 10;
	var tipLeft = mousePosition.x - (p_tip.clientWidth / 2);

	/* Does the top of the tool tip go outside the window? */
	if (tipTop < 0)
		tipTop = mousePosition.y + 20; /* This is arbitrary, and could be larger */
	/* Does the left of the tool tip go outside the window? */
	if (tipLeft < 0)
		tipLeft = 0;
	/* Does the mouse pointer plus half of the tool tip go beyond the window? */
	if (mousePosition.x + (p_tip.clientWidth / 2) >= windowSize.x - 1)
		tipLeft = windowSize.x - p_tip.clientWidth - 2;
	return ({
		top: tipTop,
		left: tipLeft
	});
}

/**
 * This function, showToolTip, gets the tool tip that is to be shown to the user,
 * positions it according to the mouse position, and sets the CSS styles necessary
 * to make it visible.
 */
function showToolTip( ) {
	/* The tool-tip element to be shown to the user */
	var toolTip = $(this.id + 'Def');
	/* The position that the tool tip should be in */
	var position = positionToolTip(toolTip);

	Element.setStyle(toolTip, {
		position: 'absolute',
		display: 'block',
		left: position.left,
		top: position.top
	});
}

/**
 * This function, moveToolTip, gets the visible tool tip that is to be moved, and
 * positions it according to the mouse position.
 */
function moveToolTip( ) {
	/* The tool-tip element to be moved with the mouse pointer */
	var toolTip = $(this.id + 'Def');
	/* The position that the tool tip should be in */
	var position = positionToolTip(toolTip);

	Element.setStyle(toolTip, {
		left: position.left,
		top: position.top
	});
}

/**
 * This function, hideToolTip, gets the visible tool tip that is to be hidden, and
 * hides it.
*/
function hideToolTip( ) {
	/* The tool-tip element to be hidden */
	var toolTip = $(this.id + 'Def');

	Element.setStyle(toolTip, {
		display: 'none'
	});
}

/**
 * This function, loadToolTips, handles the initial setup of the tool tips by
 * setting event handlers for each of them and getting the initial size of the
 * browser window. This function is best called on the document.onload event.
 */
function loadToolTips( ) {
	/* A list of elements that are the jumping points for the tool-tip elements */
	var elements = document.getElementsByClassName('toolTip');

	/* Loop through the list of elements and set event handlers for each of them */
	for (var i = elements.length; i > 0; i--) {
		try {
			Event.observe(elements[(i - 1)], 'mouseover', showToolTip, false);
			Event.observe(elements[(i - 1)], 'mouseout', hideToolTip, false);
			Event.observe(elements[(i - 1)], 'mousemove', moveToolTip, false);
		} catch (ex) {}
	}
	/* Get the initial size of the window */
	getResizedWindow( );
}