/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview Example 11-3. cookie.js: A simple cookie object.
 *
 * This file, cookie.js, contains a simple cookie object that can be used to get and
 * set a cookie as well as erase cookies and check to see if a given browser even
 * supports cookies.
 */

/**
 * This object, Cookie, is simply a mechanism to allow for easier access of the
 * document.cookie object for the page. It contains the following methods:
 * - set(p_name, p_value, p_expires)
 * - get(p_name)
 * - erase(p_name)
 * - accept( )
 */
var Cookie = {
	/**
	 * This method, set, creates a cookie with the name equal to /p_name/ with a
	 * value of /p_value/ that expires at the specified /p_expires/ should it
	 * exist, returning whether the cookie was created or not.
	 *
	 * @member Cookie
	 * @param {String} p_name The name for the cookie to be set.
	 * @param {String} p_value The value for the cookie to be set.
	 * @param {Float} p_expires The time before the cookie to be set expires.
	 * @return Returns whether the cookie was set or not.
	 * @type Boolean
	 */
	set: function(p_name, p_value, p_expires) {
		/* The expires string for the cookie */
		var expires = '';

		/* Was an expires time sent to the method? */
		if (p_expires != undefined) {
			/* Get a base time for the expiration date */
			var expirationDate = new Date( );

			/* Set the expiration to one day times the passed /p_expires/ value */
			expirationDate.setTime(expirationDate.getTime( ) + (86400000 * parseFloat(p_expires)));
			/* Create the expires string for the cookie */
			expires = '; expires=' + expirationDate.toGMTString( );
		}
		return (document.cookie = escape(name) + '=' + escape(p_value || '') + expires);
	},
	/**
	 * This method, get, returns the cookie with a name equal to the passed /p_name/
	 * variable if one exists.
	 *
	 * @member Cookie
	 * @param {String} p_name The name for the cookie to return.
	 * @return Returns the cookie data if it exists or /null/ otherwise.
	 * @type String
	 */
	get: function(p_name) {
		/* Get the matching cookie */
		var cookie = document.cookie.match(new RegExp('(^|;)\\s*' + escape(p_name) + '=([^;\\s]*)'));

		return (cookie ? unescape(cookie[2]) : null);
	},
	/**
	 * This method, erase, removes the cookie with the passed /p_name/ variable from
	 * the document. It returns the erased cookie (i.e., null if erase succeeded,
	 * the cookie otherwise).
	 *
	 * @member Cookie
	 * @param {String} p_name The name for the cookie to erase.
	 * @return Returns the cookie after it is erased.
	 * @type Boolean | String
	 */
	erase: function(p_name) {
		/* Get the cookie with the passed /p_name/ variable */
		var cookie = Cookie.get(p_name) || true;

		/*
		 * Set the cookie with the passed /p_name/ variable to an empty string, and
		 * make it expire
		 */
		Cookie.set(p_name, '', -1);
		return (cookie);
	},
	/**
	 * This method, accept, tests to see if the browser accepts cookies and returns
	 * the results of this test.
	 *
	 * @member Cookie
	 * @return Returns whether the browser accepts cookies or not.
	 * @type Boolean
	 */
	accept: function( ) {
		/* Can the test be accomplished using the browser's built-in members? */
		if (typeof navigator.cookieEnabled == 'boolean')
			return (navigator.cookieEnabled);
		/* Attempt to set and erase a cookie and return the results */
		Cookie.set('_test', '1');
		return (Cookie.erase('_test') === '1');
	}
};