/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */

/*
 * Example 11-4. Using cookies to store user choices incorporated in our original
 * style switcher.
 */
/**
 * This object, StyleSwitcher, contains all of the functionality to get and set an
 * alternate style chosen by the user from a list provided by the application. It
 * contains the following methods:
 * - setActive(p_title)
 * - getActive( )
 * - getPreferred( )
 * - loadStyle( )
 */
var StyleSwitcher = {
	/**
	 * This method, setActive, takes the passed /p_title/ variable and sets
	 * the appropriate alternate style sheet as the current enabled one. It then
	 * stores this choice into a cookie for future use.
	 *
	 * @member StyleSwitcher
	 * @param {String} p_title The title that is to be set as active.
	 * @return Returns false so that the click event will be ignored.
	 * @type Boolean
	 * @requires Cookie This method uses the Cookie object to store the
	 * 		user's selection.
	 * @see Cookie#set
	 */
	setActive: function(p_title) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list, setting the appropriate <link> elements to
		 * disabled, and set the <link> element with the title attribute equal to
		 * /p_title/ to active.
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];

			/* Is this element an appropriate stylesheet to mark? */
			if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('title')) {
				iLink.disabled = true;
				/* Is this element the one we are looking for? */
				if (iLink.getAttribute('title') == p_title)
					iLink.disable = false;
			}
		}
		/*
		 * Set the cookie to the passed /p_title/ variable, and do not let it expire
		 * for one year
		 */
		Cookie.set('appStyle', p_title, 365);
		return (false);
	},
	/**
	 * This method, getActive, returns the current active stylesheet node (<link>
	 * element) for the page, provided that there is one to return.
	 *
	 * @member StyleSwitcher
	 * @return Returns the active stylesheet node, if one exists.
	 * @type Node
	 */
	getActive: function( ) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list until the active stylesheet is located, then
		 * return it
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];

			/* Is this the currently active <link> element? */
			if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('title') && !iLink.disabled)
				return (iLink.getAttribute('title'));
		}
		return (null);
	},
	/**
	 * This method, getPreferred, returns the preferred stylesheet title for the
	 * page, provided that there is one to return.
	 *
	 * @member StyleSwitcher
	 * @return Returns the preferred stylesheet title, if one exists.
	 * @type String
	 */
	getPreferred: function( ) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list until the preferred stylesheet is located, then
		 * return it.
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];

			/* Is this the preferred <link> element? */
			if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('rel').indexOf('alt') == -1 && iLink.getAttribute('title'))
				return (iLink.getAttribute('title'));
		}
		return (null);
	},
	/**
	 * This method, loadStyle, loads the stylesheet for the application,
	 * attempting to first get it from the cookie, and if not from there, then the
	 * preferred stylesheet for the page is selected instead.
	 *
	 * @member StyleSwitcher
	 * @requires Cookie This method uses the Cookie object to get the
	 * 		user's selection.
	 * @see Cookie#get
	 */
	loadStyle: function( ) {
		/* Get the cookie, and extract the appropriate title to set active */
		var cookie = Cookie.get('appStyle');
		var title = ((cookie) ? cookie : this.getPreferred( ));

		/* Set the active stylesheet for the page */
		this.setActive(title);
		}
};

try {
	/* Load the style sheet for the page */
	Event.observe(window, 'load', StyleSwitcher.loadStyle, false);
} catch (ex) {}