/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */

/**
 * @fileoverview Example 11-6. styleSwitcher.js: The modified version of the
 * /styleSwitcher/ object.
 *
 * This file, styleSwitcher.js, contains the styleSwitcher object, which
 * represents one instance of a group of styles that the user can switch between.
 */

/**
 * This object, StyleSwitcher, contains all of the functionality to get and set
 * an alternative style chosen by the user from a list provided by the application.
 * It contains the following methods:
 * 		- initialize(p_cookieName)
 * 		- setActive(p_title)
 * 		- getActive( )
 * 		- getPreferred( )
 * 		- loadStyle( )
 * This class requires that the titles of the stylesheets be of the form
 * /group/_/value/ so that it is easier to determine the groups each belongs to.
 * The preferred stylesheet for a group should have a title of /group/ - and the
 * cookie name should also be /group/.
 */
var styleSwitcher = Class.create( );
styleSwitcher.prototype = {
	/**
	 * This member, _cookieName, stores the name of the cookie for later
	 * use in the object.
	 * @private
	 */
	_cookieName: null,
	/**
	 * This member,_multiple, lets the object know whether it should be
	 * switching multiple files or just single files.
	 * @private
	 */
	_multiple: false,
	/**
	 * This method, initialize, is the constructor for the class. The cookie
	 * name for the individual objects should be the group name.
	 *
	 * @member styleSwitcher
	 * @constructor
	 * @param {String} p_cookieName The name of the /group/ and the cookie.
	 * @param {Boolean} p_multiple An optional parameter to tell the object
	 * 		whether multiple CSS files are involved or not.
	 */
	initialize: function(p_cookieName, p_multiple) {
		this._cookieName = p_cookieName;
		if (p_multiple != 'undefined' && typeof p_multiple == 'boolean')
			this._multiple = p_multiple;
		this.loadStyle( );
	},
	/**
	 * This method, setActive, takes the passed /p_title/ variable and sets
	 * the appropriate alternate stylesheet as the currently enabled one. It then
	 * stores this choice into a cookie for future use.
	 *
	 * @member styleSwitcher
	 * @param {String} p_title The title that is to be set as active.
	 * @return Returns false so that the click event will be ignored.
	 * @type Boolean
	 * @requires Cookie This method uses the Cookie object to store the user's
	 * 		selection.
	 * @see Cookie#set
	 */
	setActive: function(p_title) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list, setting the appropriate <link> elements to
		 * disabled, and set the <link> element with the title attribute equal to
		 * /p_title/ to active.
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];

			/* Is this element an appropriate stylesheet to mark? */
			if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('title')) {
				iLink.disabled = true;
				if (!this._multiple) {
					/* Is this element the one we are looking for? */
					if (iLink.getAttribute('title') == p_title)
						iLink.disable = false;
				} else {
					var underscoreCount = group.match(/_/g);
					var titleName = group.split('_');

					if ((underscoreCount == 1 && titleName[0] == p_title) || (underscoreCount == 2 && (titleName[0] + '_' + titleName[1]) == p_title))
						iLink.disable = false;
				}
			}
		}
		/*
		 * Set the cookie to the passed /p_title/ variable, and do not let it
		 * expire for one year
		 */
		Cookie.set(this._cookieName, p_title, 365);
		return (false);
	},
	/**
	 * This method, getActive, returns the current active stylesheet node (<link>
	 * element) for the page, provided that there is one to return.
	 *
	 * @member styleSwitcher
	 * @return Returns the active stylesheet node, if one exists.
	 * @type Node
	 */
	getActive: function( ) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list until the active stylesheet is located, then
		 * return it.
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];

			/* Is this the currently active <link> element? */
			if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('title') && !iLink.disabled)
				if (!this._multiple) {
					return (iLink.getAttribute('title'));
				} else {
					var underscoreCount = group.match(/_/g);
					var titleName = group.split('_');
			
					if (underscoreCount == 1)
						return (titleName[0]);
					else
						return (titleName[0] + '_' + titleName[1]);
				}
		}
		return (null);
	},
	/**
	 * This method, getPreferred, returns the preferred stylesheet title for
	 * the page, provided that there is one to return.
	 *
	 * @member styleSwitcher
	 * @return Returns the preferred stylesheet title, if one exists.
	 * @type String
	 */
	getPreferred: function( ) {
		/* Get a list of the <link> elements in the document */
		var links = document.getElementsByTagName('link');

		/*
		 * Loop through the list until the preferred stylesheet is located,
		 * then return it.
		 */
		for (var i = links.length; i > 0;) {
			/* Get the current <link> element */
			var iLink = links[i--];
			var group = iLink.getAttribute('title');

			if (!this._multiple) {
				if (group.indexOf('_') == -1 && group == this._cookieName)
				/* Is this the preferred <link> element? */
				if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('rel').indexOf('alt') == -1)
					return (group);
			} else {
				var underscoreCount = group.match(/_/g);
				var titleName = group.split('_');

				if (underscoreCount == 1 && titleName[0] == this._cookieName)
					/* Is this the preferred <link> element? */
					if (iLink.getAttribute('rel').indexOf('style') != -1 && iLink.getAttribute('rel').indexOf('alt') == -1)
						return (titleName[0]);
			}
		}
		return (null);
	},
	/**
	 * This method, loadStyle, loads the stylesheet for the application,
	 * attempting to first get it from the cookie, and if not from there, then the
	 * preferred stylesheet for the page is selected instead.
	 *
	 * @member styleSwitcher
	 * @requires Cookie This method uses the Cookie object to get the user's
	 * 		selection.
	 * @see Cookie#get
	 */
	loadStyle: function( ) {
		/* Get the cookie, and extract the appropriate title to set active */
		var cookie = Cookie.get(this._cookieName);
		var title = ((cookie) ? cookie : this.getPreferred( ));

		/* Set the active style sheet for the page */
		this.setActive(title);
	}
};