/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview, Example 12-5. customError.js: A modified custom Error object
 * with user input.
 *
 * This file, customError.js, contains custom error codes, and the /myError/ object
 * to use to throw errors in the Ajax application.
 */

var ERROR_UNDEFINED = 0;
var ERROR_NOTICE = 1;
var ERROR_WARNING = 2;
var ERROR_CRITICAL = 3;
var ERROR_SILENT_WARN = 4;
var ERROR_SILENT_CRIT = 5;
var ERROR_SERVER_WARN = 6;
var ERROR_SERVER_CRIT = 7;

/**
 * This object, myError, is used to send errors where they are required. It has
 * the following public methods:
 * 		- throw(p_level, p_number, p_message, p_param)
 * 		- restart(p_method)
 *
 * @type object
 * @constructor
 */
var myError = {
	/**
	 * This member, level, holds the custom level of the error.
	 * @type Integer
	 */
	level: ERROR_UNDEFINED,
	/**
	 * This member, number, holds the number of the error.
	 * @type Integer
	 */
	number: -1,
	/**
	 * This member, message, holds the message of the error.
	 * @type String
	 */
	message: '',
	/**
	 * This member, parameters, holds the optional parameters object for the error.
	 * @type Object | null
	 */
	parameters: null,
	/* The following are constants to myError for user input on the pop up */
	FORM_EMAIL-OCCURRED: 1,
	FORM_EMAIL-INPUT, 2,
	/**
	 * This method, sendToUser, formats the error and sends it to the user using
	 * the /fillPopUp( )/ method for display.
	 *
	 * @member myError
	 * @see #parseError
	 */
	sendToUser: function( ) {
	/*
	* This variable, format, will hold the formatted error to display to
	* the user
	*/
	var format = '';
	// Decide how the error should be formatted, and do so here...
	if (this.parameters.form == this.FORM_EMAIL-OCCURRED)
	/* Create a form that has inputs for an email and what occurred */
	else if (this.parameters.form == this.FORM_EMAIL-INPUT)
	/*
	* Create a form that has inputs for an email and the input from
	* the user
	*/
	/* ... */
	/* This is from Example 10-2 in Chapter 10 */
	fillPopUp('Error', format);
	},
	/**
	* This method, sendToServer, formats the error and, depending on the
	* /p_method/ passed in, will send the error to the database for logging,
	* email the developers, or both.
	*
	* @member myError
	* @param {Integer} p_method The method to send to the server.
	* @see #parseError
	* @see Ajax#Request
	*/
	sendToServer: function(p_method) {
	/*
	* This variable, param, will hold the formatted error to send to the
	* server
	*/
	var param = '';
	param += '<error>';
	param += '<number>' + this.number + '</number>';
	param += '<message>' + this.message + '</message>';
	if (this.parameters.file)
	param += '<file>' + this.parameters.file + '</file>';
	if (this.parameters.line)
	param += '<line>' + this.parameters.line + '</line>';
	if (this.parameters.trace)
	param += '<trace>' + this.parameters.trace + '</trace>';
	param += '</error>';
	/* What method should be used? */
	switch (p_method) {
	case 1:
	Ajax.Request('logError.php', {
	method: 'post',
	parameters: param
	});
	break;
	case 2:
	Ajax.Request('emailError.php', {
	method: 'post',
	parameters: param
	});
	break;
	case 3:
	Ajax.Request('logError.php', {
	method: 'post',
	parameters: param
	});
	Ajax.Request('emailError.php', {
	method: 'post',
	parameters: param
	});
	break;
	}
	},
	/**
	* This method, throw, takes the error parameters passed in by the user, sets
	* them to the object's members, and calls the error parser.
	*
	* @member myError
	* @param {Integer} p_level The custom level of the error.
	* @param {Integer} p_number The number of the error, usually given by the
	* system.
	* @param {String} p_message The message of the error, usually given by the
	* system.
	* @param {Object} p_param Optional object containing additional parameters
	* to send.
	* @see #parseError
	*/
	throw: function(p_level, p_number, p_message, p_param) {
	this.level = p_level;
	this.number = p_number & 0xFFFF;
	this.message = p_message;
	this.parameters = p_param;
	this.parseError( );
	},
	/**
	* This member, parseError, looks at the custom level of the error and
	* determines where the error should be sent.
	*
	* @member myError
	* @see #throw
	*/
	parseError: function( ) {
	/* What is the level of the error? */
	switch (this.level) {
	case ERROR_NOTICE:
	this.sendToUser( );
	break;
	case ERROR_WARNING:
	case ERROR_SILENT_WARN:
	this.sendToServer(1);
	if (this.level != ERROR_SILENT_WARN)
	this.sendToUser( );
	break;
	case ERROR_CRITICAL:
	case ERROR_SILENT_CRIT:
	this.sendToServer(3);
	if (this.level != ERROR_SILENT_WARN)
	this.sendToUser( );
	this.restart(1);
	break;
	}
	},
	/**
	* This member, restart, restarts the application either when the custom
	* pop-up window has closed or regardless of what is happening in the
	* application.
	*
	* @member myError
	* @param {Integer} p_method Lets the object know if the error should be sudden.
	*/
	restart: function(p_method) {
	/* Do we care if anything is going on and is the pop up visible? */
	if (p_method && Element.visible('popupContainer'))
	/* check again in a quarter of a second */
	setTimeout('myError.restart(1)', 250);
	/* Can we just restart? */
	else if (!p_method || !Element.visible('popupContainer'))
	window.location.href = window.location.href;
	}
}