/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, pngAnimation.js, encapsulates all of the logic and code
 * needed to take a PNG image that has "frames" and animate it according to the
 * developer's designs. To allow for the greatest flexibility, the /animation/
 * class contains an internal timer so that multiple instances of the object will
 * each have their own timing mechanism.
 */

/**
 * This class, animation, creates the illusion of animation with a PNG image while
 * allowing the developer the ability to control certain aspects of the animation.
 * It contains the following methods:
 * 		- initialize(p_id, p_frameSize, p_frameCount, p_pauseTime)
 * 		- advanceFrame()
 */
var animation = Class.create();
animation.prototype = {
	/**
	 * This member, _handle, stores the <div> element (presumably) that stands in
	 * for the image.
	 * @private
	 */
	_handle: null,
	/**
	 * This member, _frameSize, stores the width of an individual "frame" in the
	 * image.
	 * @private
	 */
	_frameSize: 0,
	/**
	 * This member, _frameCount, stores the number of "frames" contained in the
	 * image.
	 * @private
	 */
	_frameCount: 0,
	/**
	 * This member, _pauseTime, stores the length of time that the animation
	 * should pause between "frames" kept in milliseconds.
	 * @private
	 */
	_pauseTime: 0,
	/**
	 * This member, _currentFrame, stores the "frame" currently being viewed in
	 * the browser.
	 * @private
	 */
	_currentFrame: 0,
	/**
	 * This member, _internalTimer, stores the switching time for the object.
	 * @private
	 */
	_internalTimer: null,
	/**
	 * This method, initialize, is the constructor for the class and sets all of
	 * the necessary private members before starting the animation.
	 *
	 * @member animation
	 * @constructor
	 * @param {String | Object} p_id The id or object used for the animation.
	 * @param {Integer} p_frameSize The width of an individual "frame" in the image.
	 * @param {Integer} p_frameCount The number of "frames" in the image.
	 * @param {Integer} p_pauseTime The time (in milliseconds) the animation
	 * 		should pause between "frames".
	 */
	initialize: function(p_id, p_frameSize, p_frameCount, p_pauseTime) {
		/* Set all of the private members */
		this._handle = $(p_id);
		this._frameSize = p_frameSize;
		this._frameCount = p_frameCount;
		this._pauseTime = p_pauseTime;
		/*
		 * Start the animation. By using the prototype bind method, each instance
		 * of this object can have its own timera very useful feature.
		 */
		this._internalTimer = setInterval(this.advanceFrame.bind(this), this._pauseTime);
	},
	/**
	 * This member, advanceFrame, changes the position of the background image of
	 * the /_handle/ based on the /_currentFrame/ and /_frameSize/.
	 *
	 * @member animation
	 */
	advanceFrame: function() {
		/* Should the animation start over at the beginning? */
		if (this._currentFrame == this._frameCount)
			this._currentFrame = 0;
		/*
		 * Move the background image to advance the "frame", then change the
		 * /_currentFrame/
		 */
		this._handle.style.backgroundPosition = (this._frameSize * this._currentFrame * -1) + 'px 0';
		this._currentFrame++;
	}
};