/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, pngAnimation.js, encapsulates all of the logic and code
 * needed to take a PNG image that has "frames" and animate it according to the
 * developer's designs. To allow for the greatest flexibility, the /animation/
 * class contains an internal timer so that multiple instances of the object will
 * each have their own timing mechanism.
 *
 * This code requires the Prototype framework.
 */

/**
 * This class, animation, creates the illusion of animation with a PNG image while
 * allowing the developer the ability to control certain aspects of the animation.
 * It contains the following methods:
 * 		- initialize(p_id, p_frameSize, p_frameCount, p_pauseTime)
 * 		- advanceFrame()
 * 		- startAnimation()
 * 		- pauseAnimation()
 * 		- stopAnimation()
 */
var animation = Class.create();
animation.prototype = {
	/**
	 * This member, _handle, stores the <div> element (presumably) that stands in
	 * for the image.
	 * @private
	 */
	_handle: null,
	/**
	 * This member, _frameSize, stores the width of an individual "frame" in the
	 * image.
	 * @private
	 */
	_frameSize: 0,
	/**
	 * This member, _frameCount, stores the number of "frames" contained in the
	 * image.
	 * @private
	 */
	_frameCount: 0,
	/**
	 * This member, _pauseTime, stores the length of time that the animation should
	 * pause between "frames" kept in milliseconds.
	 * @private
	 */
	_pauseTime: 0,
	/**
	 * This member, _currentFrame, stores the "frame" currently being viewed in the
	 * browser.
	 * @private
	 */
	_currentFrame: 0,
	/**
	 * This member, _internalTimer, stores the switching time for the object.
	 * @private
	 */
	_internalTimer: null,
	/**
	 * This member, _poll, stores the object parameter that is passed on
	 * /initialize/.
	 * @private
	 */
	_poll: null,
	/**
	 * This method, initialize, is the constructor for the class and sets all of
	 * the necessary private members.
	 *
	 * @member animation
	 * @constructor
	 * @param {String | Object} p_id The id or object used for the animation.
	 * @param {Integer} p_frameSize The width of an individual "frame" in the image.
	 * @param {Integer} p_frameCount The number of "frames" in the image.
	 * @param {Integer} p_pauseTime The time (in milliseconds) the animation
	 * 		should pause between "frames".
	 */
	initialize: function(p_id, p_frameSize, p_frameCount, p_pauseTime, p_object) {
		/* Set all of the private members */
		this._handle = $(p_id);
		this._frameSize = p_frameSize;
		this._frameCount = p_frameCount;
		this._pauseTime = p_pauseTime;

		/* Was an object parameter passed? */
		if (p_object) {
			this._poll = p_object;

			/* Is there a /polling/ property? */
			if (!this._poll.polling)
				this._poll.polling = false;
			/* Is there a /callingPage/ property? */
			if (!this._poll.callingPage)
				this._poll.callingPage = false;
			/* Is there a /parameters/ property? */
			if (!this._poll.parameters)
				this._poll.parameters = '';
			/* Is there a /pollTime/ property? */
			if (!this._poll.pollTime)
				this._poll.pollTime = 5;
			this._poll.animationIteration = 0;
		}
	},
	/**
	 * This member, advanceFrame, changes the position of the background image of
	 * the /_handle/ based on the /_currentFrame/ and /_frameSize/. If polling is
	 * required, this member will make an Ajax call to the server that will
	 * retrieve more information.
	 *
	 * @member animation
	 */
	advanceFrame: function() {
		/* Should the animation start over at the beginning? */
		if (this._currentFrame == this._frameCount) {
			this._currentFrame = 0;

			/* Is polling requested? */
			if (this._poll.polling) {
				/* Is it time to make a poll to the server? */
				if ((this._poll.animationIteration % this._poll.pollTime) == 0) {
					new Ajax.Request(this._poll.callingPage, {
						method: 'post',
						parameters: this._poll.parameters,
						onSuccess: function(xhrResponse) {
							this._pauseTime.bind(this) = xhrResponse.responseText;
						}
					});
				}
				this._poll.animationIteration++;
			}
		}
		
		/*
		 * Move the background image to advance the "frame", then change the
		 * /_currentFrame/
		 */
		this._handle.style.backgroundPosition = (this._frameSize * this._currentFrame * -1) + 'px 0';
		this._currentFrame++;
	},
	/**
	 * This member, startAnimation, calls the DOM function /setInterval/ to start
	 * the timer for the animation and will report its success.
	 *
	 * @member animation
	 * @return Whether or not the animation was started.
	 * @type Boolean
	 * @see advanceFrame
	 */
	startAnimation: function() {
		/*
		 * Start the animation. By using the Prototype bind method, the
		 * /setInterval/ function will be pointed to the object's /_internalTimer/
		 * allowing each instance of this object to have its own timer-a very
		 * useful feature.
		 */
		this._internalTimer = setInterval(this.advanceFrame.bind(this), this._pauseTime);

		/* Was the timer set? */
		if (this._intervalTimer)
			return (true);
		return (false);
	},
	/**
	 * This member, pauseAnimation, calls the DOM function /clearInterval/ to clear
	 * the timer for the animation and stop it in its current frame.
	 *
	 * @member animation
	 * @return Whether or not the animation was correctly paused.
	 * @type Boolean
	 */
	pauseAnimation: function() {
		/*
		 * By using the Prototype bind method the /clearInterval/ function will
		 * clear the appropriate timer value, namely /this/ one.
		 */
		clearInterval(this._internalTimer.bind(this));

		/* Was the timer cleared? */
		if (!this._internalTimer) {
			this._internalTimer = null;
			return (true);
		}
		return (false);
	},
	/**
	 * This member, stopAnimation, calls the DOM function /clearInterval/ to clear
	 * the timer for the animation, then the /_currentFrame/ is reset to 0 and the
	 * image reset to its first "frame".
	 *
	 * @member animation
	 * @return Whether or not the animation was correctly stopped.
	 * @type Boolean
	 */
	stopAnimation: function() {
		/*
		 * By using the Prototype bind method, the /clearInterval/ function will
		 * clear the appropriate timer value, namely /this/ one.
		 */
		clearInterval(this._internalTimer.bind(this));

		/* Was the timer cleared? */
		if (!this._internalTimer) {
			this._currentFrame = 0;
			this._internalTimer = null;
			/* Move the background image to the first "frame" */
			this._handle.style.backgroundPosition = '0 0';
			return (true);
		}
		return (false);
	}
};

var myAnimation = new animation('myWrapper', 50, 8, 150, {
	polling: true,
	callingpage: 'myServerPage.php',
	parameters: 'myParam=someData',
	pollTime: 4
});