/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, fa_stats.js, requests statistical data from the server-
 * based on the choice of the user as to what data to view. Once returned, the
 * callback function to the Ajax request creates a chart using Walter Zorn's Vector
 * Graphics JavaScript Library and displays it to the user.
 */
/*
 * This variable will hold the drawing object for the global canvas that will
 * be used
 */
var barDoc = null;

/**
 * This function, body_onload, instantiates the /jsGraphics/ object to the global
 * /barDoc/ variable.
 */
function body_onload() {
	/* Define the global canvas to draw the bar graphs on */
	barDoc = new jsGraphics('chartCanvas');
}

/**
 * This function, getChart, takes the user's drop-down choice and makes a request
 * to the server for the resulting data from that choice. It then draws a bar
 * graph for the user to view based on the XML results that are returned to the
 * client.
 *
 * @return Returns false so that the element that had the event click stops any
 * 		default events.
 * @type Boolean
 * @see Ajax#Request
 */
function getChart( ) {
	/* Get the user's choice */
	var statChoice = $F('stat');

	/* Clear any bar graph that may exist from a previous call */
	barDoc.clear( );
	/* Call fa_stats.php with the user's choice */
	new Ajax.Request('fa_stats.php', {
		method: 'post',
		postBody: 'stat=' + statChoice,
		/**
		 * This method, onSuccess, is the callback method for the Ajax object when
		 * a request to the server returns successfully. Once the response is
		 * received from the server, the client draws the corresponding bar graph
		 * to the data that is returned in an XML document.
		 *
		 * @param {Object} xhrResponse The response object from the server.
		 */
		onSuccess: function(xhrResponse) {
			/* Get the XML document */
			var response = xhrResponse.responseXML;

			/* Was there an error on the server side? */
			if (response.getElementsByTagName('error').length) {
				barDoc.setColor('#000000');
				barDoc.drawString('There was a problem retrieving the data.', 100, 220);
			} else {
				/* Draw the x and y axes first */
				barDoc.setColor('#000000');
				barDoc.setStroke(2);
				barDoc.drawPolyline([30, 30, 800], [20, 400, 400])
				barDoc.paint();

				/* Get the list of stats */
				var stats = response.getElementsByTagName('stat');

				barDoc.setStroke(1);
				/*
				 * Loop through the stats and build the corresponding bars of
				 * the graph
				 */
				for (var i = 0, il = stats.length; i < il; i++) {
					/*
					 * Each bar should be 70 pixels wide and have 5 pixels of space
					 * between each
					 */
					var x = 40 + (i * 70) + (i * 5);
					/*
					 * Make the height of each bar 5 times bigger so it is easier
					 * to see
					 */
					var height = stats[i].getAttribute('height') * 5;
					/* Set the starting spot for the rectangle */
					var y = 400 - height;

					/* Draw the bar and label the axis and value for each */
					barDoc.setColor(stats[i].getAttribute('color'));
					barDoc.fillRect(x, y, 70, height);
					barDoc.setFont('Arial', '14px', Font.BOLD);
					barDoc.setColor('#000000');
					barDoc.drawString(stats[i].getAttribute('id'), x + 15, 410);
					barDoc.drawString(stats[i].getAttribute('height'), x + 25, y - 20);
				}
			}
			/* Paint to the canvas whatever was drawn */
			barDoc.paint( );
			/* Display the canvas to the user */
			$('chartCanvas').style.display = 'block';
		},
		/**
		 * This method, onFailure, is the callback method for the Ajax object when
		 * a request to the server returns unsuccessfully. Once the response is
		 * received from the server, the client notifies the user of the problem.
		 *
		 * @param {Object} xhrResponse The response object from the server.
		 */
		onFailure: function(xhrResponse) {
			/* Let the user know there was a problem */
			barDoc.setColor('#000000');
			barDoc.drawString('There was a problem connecting to the server.', 100, 220);
			barDoc.paint();
			$('chartCanvas').style.display = 'block';
		}
	});
	/* Return false so that the links do not try to actually go somewhere */
	return (false);
}