/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/*
 * Example 14-5. A custom radio button and checkbox form control.
 */

/* Create custom globals to define the types of control */
var CUSTOM_RADIO = 0;
var CUSTOM_CHECKBOX = 1;

/**
 * This object, customRadioCheckControl, gives the developer a custom control for
 * building radio buttons and checkboxes that give all of the functionality of
 * these form controls without being bound to the default controls' constraints.
 */
var customRadioCheckControl = Class.create();
customRadioCheckControl.prototype = {
	/**
	 * This member, _type, defines the type of custom control this control is.
	 * @member customRadioCheckControl
	 * @type Integer
	 */
	_type: CUSTOM_RADIO,
	/**
	 * This member, _image, holds the preloaded image for the control.
	 * @member customRadioCheckControl
	 * @type Object
	 */
	_image: null,
	/**
	 * This member, checked, allows the developer to know whether the control has
	 * been checked or not.
	 * @member customRadioCheckControl
	 * @type Boolean
	 */
	checked: false,
	/**
	 * This member, _options, is the set of properties that further defines the
	 * custom control.
	 * @member customRadioCheckControl
	 * @type Object
	 */
	_options: null,
	/**
	 * This member, _displayed, defines whether or not that control has been
	 * displayed to the user.
	 * @member customRadioCheckControl
	 * @type Boolean
	 */
	_displayed: false,
	/**
	 * This method, initialize, is the constructor for the object and initializes
	 * it so that it is ready to be displayed to the user when called upon by the
	 * developer.
	 *
	 * @member customRadioCheckControl
	 * @param {Integer} p_type The type of control that is to be created.
	 * @param {String} p_src The src of the image to be used by the control to
	 * 		display the radio button or checkbox.
	 * @param {Object} p_options An object of options to further define the custom
	 * 		control.
	 * @see #_setOptions
	 */
	initialize: function(p_type, p_src, p_options) {
		this._type = p_type;
		this._setOptions(p_options);
		/* Preload the image for faster load times */
		this._image = new Image(this._options.width, this._options.height);
		this._image.src = p_src;
		this.checked = ((this._options.defaultValue == 'checked') ? true : false);
	},
	/**
	 * This method, _setOptions, takes the passed /p_options/ object and sets these
	 * values to the control's _options member.
	 *
	 * @param {Object} p_options An object of options to further define the custom
	 * 		control.
	 * @see #initialize
	 * @see Object#extend
	 */
	_setOptions: function(p_options) {
		this._options = {
			id: 'customRadioCheck',
			label: '',
			defaultValue: 'unchecked',
			state: 'enabled',
			width: 20,
			height: 20,
			onClick: null,
			onChange: null
		};
		Object.extend(this._options, p_options || {});
	},
	/**
	 * This method, _positionImage, moves the custom control's image by the amount
	 * needed to display the correct state of the control.
	 *
	 * @member customRadioCheckControl
	 * @see #initialize
	 * @see #_toggleValue
	 */
	_positionImage: function() {
		/* Is the state of the control /enabled/? */
		if (this._options.state == 'enabled')
			$(this._options.id + '_img').style.backgroundPosition = ((this.checked) ? (-1 * this._options.width) : 0) + 'px 0';
		else
			$(this._options.id + '_img').style.backgroundPosition = (-1 * ((2 *
		this._options.width) + ((this.checked)) ?
		this._options.width : 0)) + 'px 0';
	},
	/**
	 * This method, _toggleValue, overrides what the toggle value's Boolean value
	 * should be.
	 *
	 * @member customRadioCheckControl
	 * @param {Boolean} p_value The optional value to set the control to.
	 * @see #_positionImage
	 * @see #onChange
	 */
	_toggleValue: function(p_value) {
		/* Was a /p_value/ passed to the method? */
		if (p_value)
			this.checked = p_value;
		else
			this.checked = !this.checked;
		this._positionImage();
		this.onChange();
	},
	/**
	 * This method, _createEvents, sets an /onclick/ event on the custom control.
	 *
	 * @member customRadioCheckControl
	 * @see Event#observe
	 */
	_createEvents: function() {
		/* Was an id passed? */
		if (this._options.id)
			Event.observe($(this._options.id), 'click', this.onClick.bind(this), false);
	},
	/**
	 * This method, onClick, is the event handler for the /onclick/ event on the
	 * control. It toggles the value of the control and calls the developer-
	 * defined callback, if one exists, passing it the object.
	 *
	 * @member customRadioCheckControl
	 * @see #_toggleValue
	 * @see Prototype#emptyFunction
	 */
	onClick: function() {
		this._toggleValue();
		try {
			(this._options['onClick'] || Prototype.emptyFunction)(this);
		} catch (ex) {
			/* An exception handler could be called here */
		}
	},
	/**
	 * This method, onChange, is the event handler for the /onchange/ event on the
	 * control. It is fired when the object has been toggled, and calls the
	 * developer-defined callback, if one exists, passing it the object.
	 *
	 * @member customRadioCheckControl
	 * @see #_toggleValue
	 * @see Prototype#emptyFunction
	 */
	onChange: function() {
		try {
			(this._options['onChange'] || Prototype.emptyFunction)(this);
		} catch (ex) {
			/* An exception handler could be called here */
		}
	},
	/**
	 * This method, reset, allows the developer to reset the control to its
	 * original state.
	 *
	 * @member customRadioCheckControl
	 * @see #_toggleValue
	 */
	reset: function() {
		this.checked = ((this._options.defaultValue == 'checked') ? true : false);
		this._toggleValue(this.checked);
	},
	/**
	 * This method, display, is the method called by the developer when and where
	 * it is to be placed into the page document. Once displayed, it cannot be
	 * displayed again, but only reset.
	 *
	 * @member customRadioCheckControl
	 * @see #createEvents
	 */
	display: function() {
		/* Has the control been displayed already? */
		if (!this._displayed) {
			/*
			 * This will be the variable that will hold the display for the
			 * control
			 */
			var control = '';

			control += '<div id="' + this._options.id + '" class="customRadioCheck">';
			control += '<div id="' + (this._options.id + '_img') + '"></div>';
			/* Is there a label? */
			if (this._options.label)
				control += '<div class="label">' + this._options.label + '</div>';
			control += '</div>';
			/* Place the control in the document */
			document.write(control);
			/* Configure the check box or radio button */
			$(this._options.id + '_img').style.overflow = 'hidden';
			$(this._options.id + '_img').style.width = this._options.width + 'px';
			$(this._options.id + '_img').style.height = this._options.height + 'px';
			$(this._options.id + '_img').style.background = 'url(\'' + this._image.src + '\') no-repeat';

			/* Position the image where it needs to be */
			this._positionImage( );

			/* Is there a label? */
			if (this._options.label) {
				/* Position the label next to the checkbox or radio button */
				$(this._options.id).childNodes[1].style.paddingLeft = (this._options.width + 5) + 'px';
				$(this._options.id).childNodes[1].style.marginTop = (-1 * (this._options.height - 2)) + 'px';
			}
			$(this._options.id).style.display = 'inline';
			this._createEvents();
			this._displayed = true;
		}
	}
};