/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/*
 * Example 14-6. A custom drop-down object.
 */

/**
 * This object, customDropDownControl, gives the developer a custom control for
 * building drop-down controls that give all of the functionality of this form
 * control without being bound to the default control's constraints.
 *
 * This control requires the Prototype Framework Version 1.5.0 or higher.
 */
var customDropDownControl = Class.create();
customDropDownControl.prototype = {
	/**
	 * This member, _options, is the set of properties that further define the
	 * custom control.
	 * @member customDropDownControl
	 * @type Object
	 */
	_options: null,
	/**
	 * This member, _selected, holds the text value of the selected option.
	 * @member customDropDownControl
	 * @type String
	 */
	_selected: '',
	/**
	 * This member, _src, holds the string for the image controlling the custom
	 * control.
	 * @member customDropDownControl
	 * @type String
	 */
	_src: '',
	/**
	 * This member, _value, holds the current value of the custom control.
	 * @member customDropDownControl
	 * @type Mixed
	 */
	_value: '',
	/**
	 * This method, initialize, is the constructor for the object and initializes
	 * it so that it is ready to be displayed to the user when called upon by the
	 * developer.
	 *
	 * @member customDropDownControl
	 * @constructor
	 * @param {String} p_src The name of the image file source for the control.
	 * @param {Object} p_options An object of options to further define the
	 * 		custom control.
	 * @see #setOptions
	 */
	initialize: function(p_src, p_options) {
		this._setOptions(p_options);
		this._src = p_src;
	},
	/**
	 * This member, _setOptions, takes the passed /p_options/ object and sets these
	 * values to the control's _options member.
	 *
	 * @member customDropDownControl
	 * @param {Object} p_options An object of options to further define the custom
	 * 		control.
	 * @see #initialize
	 * @see Object#extend
	 */
	_setOptions: function(p_options) {
		this._options = {
			background: '#fff',
			border: '#000',
			color: '#000',
			height: 20,
			highlight: '#ccc',
			id: 'customDropDown',
			label: '',
			onClick: null,
			onChange: null,
			state: 'up',
			width: 20
		};
		Object.extend(this._options, p_options || {});
	},
	/**
	 * This method, _positionImage, moves the custom control's image by the amount
	 * needed to display the correct state of the control.
	 *
	 * @member customDropDownControl
	 * @see #initialize
	 * @see #changeValue
	 * @see #onActivate
	 * @see #display
	 * @see Element#setStyle
	 */
	_positionImage: function() {
		$(this._options.id + '_img').setStyle({
			backgroundPosition: ((this._options.state == 'down') ? (-1 * this._options.width) : 0) + 'px 0'
		});
	},
	/**
	 * This member, _changeValue, changes the value of the control to the clicked
	 * value in the control, then closes the options available.
	 *
	 * @member customDropDownControl
	 * @param {Object} e The event object that triggered this event.
	 * @see #onClick
	 * @see Element#setStyle
	 */
	_changeValue: function(e) {
		e = ((e) ? e : window.event);

		var src = ((e.target) ? e.target: e.srcElement);

		/* Is the state of the control 'down'? */
		if (this._options.state == 'down') {
			var className = src.className;
			var change = false;

			this._selected = src.innerHTML;
			if (this._value == className.substring(className.indexOf('v') + 1))
				change = true;
			this._value = className.substring(className.indexOf('v') + 1);
			$((this._options.id + '_selected')).innerHTML = this._selected;
			this._options.state = 'up';
			$((this._options.id + '_options')).setStyle({ display: 'none' });
			this._positionImage();
			$((this._options.id + '_ctrl')).setStyle({
				backgroundColor: this._options.background
			});
			if (change)
				this.onChange();
		}
	},
	/**
	 * This method, _createEvents, sets an onclick event for the control's image
	 * 'button'.
	 *
	 * @member customDropDownControl
	 * @see #display
	 * @see Event#observe
	 * @see #onActivate
	 */
	_createEvents: function() {
		/* Was an id passed? */
		if (this._options.id)
			Event.observe($((this._options.id + '_img')), 'click', this.onActivate.bind(this), false);
	},
	/**
	 * This method, _createCSS, sets all of the CSS rules for the control for the
	 * look and feel of the custom control.
	 *
	 * @member customDropDownControl
	 * @see #display
	 * @see Position#positionedOffset
	 * @see Element#getDimensions
	 * @see Element#setStyle
	 */
	_createCSS: function() {
		/* Set the CSS rules for the control's label */
		$((this._options.id + '_label')).setStyle({
			float: 'left',
			paddingRight: '5px',
			display: 'inline'
		});

		/* Get the position for where the control needs to go */
		var pos = Position.positionedOffset($((this._options.id + '_label')));
		var dims = $((this._options.id + '_label')).getDimensions();

		/* Set the CSS rules for the control itself */
		$((this._options.id + '_ctrl')).setStyle({
			backgroundColor: this._options.background,
			border: '2px inset ' + this._options.border,
			float: 'left',
			height: '1.15em',
			left: pos[0] + dims.width + 'px',
			padding: '2px 1px 2px 5px',
			position: 'absolute',
			top: (pos[1] - 4) + 'px',
			verticalAlign: 'middle',
			width: this._options.width + 'px'
		});
		/* Set the CSS rules for the control's image button */
		$((this._options.id + '_img')).setStyle({
			background: 'transparent url(\'' + this._src+ '\') no-repeat',
			float: 'left',
			height: this._options.height + 'px',
			margin: '-2px 1px 0',
			overflow: 'hidden',
			width: this._options.width + 'px'
		});
		/* Set the CSS rules for the control's selected option */
		$((this._options.id + '_selected')).setStyle({
			fontWeight: 'bold',
			margin: '0 0 0 -1.75em',
			position: 'relative'
		});

		var dims2 = $((this._options.id + '_ctrl')).getDimensions();

		/* Set the CSS rules for the control's options */
		$((this._options.id + '_options')).setStyle({
			backgroundColor: this._options.background,
			border: '1px solid ' + this._options.border,
			display: 'none',
			height: '1.15em',
			left: pos[0] + dims.width + 'px',
			overflow: 'auto',
			position: 'absolute',
			top: (pos[1] + dims2.height - 4) + 'px',
			width: (this._options.width + 8) + 'px',
			zIndex: '9000'
		});
	},
	/**
	 * This method, onActivate, opens and closes the options of the control based
	 * on their current state, also highlighting or removing the highlight on the
	 * background.
	 *
	 * @member customDropDownControl
	 * @see Element#setStyle
	 * @see Element#getElementsByClassName
	 * @see #_positionImage
	 */
	onActivate: function() {
		/* Is the current state /down/? */
		if (this._options.state == 'down') {
			this._options.state = 'up';
			$((this._options.id + '_options')).setStyle({ display: 'none' });
			$((this._options.id + '_selected')).innerHTML = this._selected;
			$((this._options.id + '_ctrl')).setStyle({
				backgroundColor: this._options.background
			});
		} else {
			this._options.state = 'down';
			$((this._options.id + '_options')).setStyle({ display: 'block' });
			$((this._options.id + '_ctrl')).setStyle({
				backgroundColor: this._options.highlight
			});

			/* Loop through the options and blank the background */
			for (var i = 0, il = $((this._options.id + '_options')).childNodes.length; i < il; i++)
				$((this._options.id + '_options')).setStyle({
					backgroundColor: this._options.background
				});

			var divElements = $((this._options.id + '_options')).childNodes[0];

			/* Does the options part of the control have options? */
			if (divElements && this._value) {
				var value = Element.getElementsByClassName($((this._options.id + '_options')), 'v' + this._value)[0];

				value.setStyle({ backgroundColor: this._options.highlight });
			}
		}
		this._positionImage();
	},
	/**
	 * This method, onClick, is called whenever one of the options is clicked on by
	 * the user, changing the value of the control and calling any user-defined
	 * function on the callback.
	 *
	 * @member customDropDownControl
	 * @param {Object} e This is the event that is calling the method.
	 * @see #_changeValue
	 * @see #addOption
	 * @see Prototype#emptyFunction
	 */
	onClick: function(e) {
		this._changeValue(e);
		try {
			(this._options['onClick'] || Prototype.emptyFunction)(this);
		} catch (ex) {
			/* An exception handler could be called here */
		}
	},
	/**
	 * This method, onChange, is called whenever the value of the control changes,
	 * calling any user-defined function on the callback.
	 *
	 * @member customDropDownControl
	 * @see #_changeValue
	 * @see Prototype#emptyFunction
	 */
	onChange: function() {
		try {
			(this._options['onChange'] || Prototype.emptyFunction)(this);
		} catch (ex) {
			/* An exception handler could be called here */
		}
	},
	/**
	 * This method, getValue, is used to get the value of the control whenever
	 * needed.
	 *
	 * @member customDropDownControl
	 */
	getValue: function() {
		return this._value;
	},
	/**
	 * This method, addOption, is used to add a new option to the control to be
	 * viewed by the user. Once the new option is added, the option and control is
	 * resized to accommodate the new option, if needed.
	 *
	 * @member customDropDownControl
	 * @param {String} p_option The text for the new option.
	 * @param {String} p_value The value for the new option.
	 * @see Element#setStyle
	 * @see Element#getDimensions
	 */
	addOption: function(p_option, p_value) {
		var newElement = document.createElement('div');
		var textOption = document.createTextNode(p_option);

		newElement.appendChild(textOption);
		newElement.setAttribute('onclick', this._options.id + '.onClick(event);');
		newElement.setAttribute('onmouseover', 'this.style.backgroundColor = \'' + this._options.highlight + '\'');
		newElement.setAttribute('onmouseout', 'this.style.backgroundColor = \'' + this._options.background + '\'');
		newElement.setAttribute('class', 'v' + p_value);
		$((this._options.id + '_options')).appendChild(newElement);

		var width = this._options.width;
		/* Create a temporary <div> element to get the dimensions, then remove it */
		var tempElement = document.createElement('div');
		var tempTextOption = document.createTextNode(p_option);

		tempElement.appendChild(tempTextOption);
		$('holding').appendChild(tempElement);
		tempElement.setStyle({ display: 'inline' });

		var dims = tempElement.getDimensions();

		$('holding').removeChild(tempElement);
		/* Should the width be changed? */
		if (dims.width > width)
			width = dims.width;

		/* Calculate the height based on the number of options */
		var height = (1.25 * $((this._options.id + '_options')).childNodes.length);

		height = ((height <= 10) ? height : 10);
		$((this._options.id + '_options')).setStyle({
			height: height + 'em',
			width: (width + 35) + 'px'
		});
		$((this._options.id + '_ctrl')).setStyle({
			width: (width + 27) + 'px'
		});
		$((this._options.id + '_img')).setStyle({
			position: 'relative',
			left: (width + this._options.width - 14) + 'px'
		});
	},
	/**
	 * This method, display, is used to initially display the control to the user;
	 * it is empty until options are added to it.
	 *
	 * @member customDropDownControl
	 * @see #_createCSS
	 * @see #_positionImage
	 * @see #_createEvents
	 */
	display: function() {
		/* Has the control been displayed already? */
		if (!this._displayed) {
			/*
			 * This will be the variable that will hold the display for the
			 * control
			 */
			var control = '';

			control += '<div id="' + this._options.id + '" class="customDropDown">';
			/* Is there a label? */
			if (this._options.label)
				control += '<div id="' + (this._options.id + '_label') + '" class="label">' + this._options.label + '</div>';
			control += '<div id="' + (this._options.id + '_ctrl') + '">';
			control += '<div id="' + (this._options.id + '_img') + '"></div>';
			control += '<span id="' + (this._options.id + '_selected') + '">' + this._selected + '</span>';
			control += '</div>';
			control += '<div id="' + (this._options.id + '_options') + '">';
			control += '</div>';
			control += '</div>';

			/* Place the control in the document */
			document.write(control);

			this._createCSS();
			this._positionImage();
			this._createEvents();
			this._displayed = true;
		}
	}
};