/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview, This file, validation.js, encapsulates some of the basic methods
 * that can be used to test values, types, and syntax from within JavaScript before
 * the form is sent to the server for processing.
 *
 * This code requires the Prototype library.
 */

/**
 * This object, Form.Validation, is an extension of the Prototype /Form/ object and
 * handles all of the methods needed for validation on the client side. It consists
 * of the following methods:
 * - isNull(p_id)
 * - isNumber(p_id)
 * - isMoney(p_id)
 * - testValue(p_id, p_value)
 * - isValidDate(p_id)
 * - isPhoneNumber
 * - isValidEmail
 * - isSSN
 * - isValidCreditCard
 */
Form.Validation = {
	/**
	 * This method, isNull, checks to see if the passed parameter /p_id/ has a
	 * value that is null or not. It also checks for empty strings, as form values
	 * cannot really be null.
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input is a valid
	 * 		number or not.
	 * @type Boolean
	 */
	isNull: function(p_id) {
		try {
			p_id = $F(p_id);
			return (p_id == null || p_id == '');
		} catch (ex) {
			return (true);
		}
	},
	/**
	 * This member, isNumber, checks to see if the passed /p_id/ has a value that
	 * is a valid number. The method can check for the following types of number:
	 * 		- 5
	 * 		- -5
	 * 		- 5.235
	 * 		- 5.904E-03
	 * 		- etc., etc., etc....(you get the idea, right?)
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input is a valid
	 * 		number or not.
	 * @type Boolean
	 */
	isNumber: function(p_id) {
		try {
			return (/^[-+]?\d*\.?\d+(?:[eE][-+]?\d+)?$/.test($F(p_id)));
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This member, isMoney, checks to see if the passed /p_id/ has a value that
	 * is a valid monetary value. The method can check for the following types of
	 * number:
	 * 		- 250
	 * 		- -250
	 * 		- 250.00
	 * 		- $250
	 * 		- $250.00
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input is a valid
	 * 		monetary value or not.
	 * @type Boolean
	 */
	isMoney: function(p_id) {
		try {
			return (/^[-+]?\$?\d*\.?\d{2}?$/.test($F(p_id)));
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, testValue, checks to see if the passed /p_id/ has a value that
	 * is equal to the passed /p_value/ in both value and type.
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @param {Number | String | Boolean | Object | Array | null | etc.} p_value
	 * 		The value to test against.
	 * @return Returns a value indicating whether the passed inputs are equal to
	 * 		one another.
	 * @type Boolean
	*/
	testValue: function(p_id, p_value) {
		try {
			return($F(p_id) === p_value);
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, isValidDate, checks to see if the passed /p_id/ has a value
	 * that is a valid /Date/. The method can check for the following date
	 * formats:
	 * 		- mm/dd/yyyy
	 * 		- mm-dd-yyyy
	 * 		- mm.dd.yyyy
	 * where /mm/ is a one- or two-digit month, /dd/ is a one- or two-digit day and
	 * /yyyy/ is a four-digit year.
	 *
	 * After the format is validated, this method checks to make sure the value is
	 * a valid date (i.e., it did or will exist.)
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input is a valid date
	 * 		or not.
	 * @type Boolean
	 */
	isDate: function(p_id) {
		try {
			date = $F(p_id);

			/* Is the value in the correct format? */
			if (!/^\d{1,2}(\/|\-|\.)\d{1,2}\1\d{4}$/.test(date))
				return (false);
			else {
				/*
				 * Find the separator for the different date parts, then split
				 * it up.
				 */
				var ds = /^\/|\-|\.$/;

				ds = date.split(ds.exec(/^\/|\-|\.$/), 3);
				/* Was there something to split? */
				if (ds != null) {
					/* Check if this date should exist */
					var m = ds[0], d = ds[1], y = ds[2];
					var td = new Date(y, --m, d);

					return (((td.getFullYear( ) == y) && (td.getMonth( ) == m) && (td.getDate( ) == d)));
				} else
					return (false);
			}
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, isPhoneNumber, checks the syntax of the passed /p_id/ and
	 * returns whether it is a valid US phone number in one of the following
	 * formats:
	 * 		- (000) 000-0000
	 * 		- (000)000-0000
	 * 		- 000-000-0000
	 * 		- 000 000 0000
	 * 		- 0000000000
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input has a valid US
	 * 		phone number format.
	 * @type Boolean
	 */
	isPhoneNumber: function(p_id) {
		try {
			return (/^\(?[2-9]\d{2}[\)-]?\s?\d{3}[\s-]?\d{4}$/.test($F(p_id)));
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, isValidEmail, indicates whether the passed /p_id/ has a valid
	 * email format.
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input has a valid
	 * 		email format.
	 * @type Boolean
	 */
	isValidEmail: function(p_id) {
		try {
			return (/^(([^<>( )[\]\\.,;:\s@\"]+(\.[^<>( )[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[(2([0-4]\d|5[0-5])|1?\d{1,2})(\.(2([0-4]\d|5[0-5])|1?\d{1,2})){3} \])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/.test($F(p_id)));
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, isSSN, checks to see if the passed /p_id/ is in a valid format
	 * and returns whether it is a valid Social Security number in one of the \
	 * following formats:
	 * 		- 000-00-0000
	 * 		- 000 00 0000
	 * 		- 000000000
	 *
	 * @member Form.Validation
	 * @param {String} p_id The name of the input field to get the value from.
	 * @return Returns a value indicating whether the passed input has a valid
	 * 		SSN format.
	 * @type Boolean
	 */
	isSSN: function(p_id) {
		try {
			if (!(/^\d{3}(\-|\s?)\d{2}\1\d{4}$/.test($F(p_id))))
				return (false);

			var temp = $F(p_id)

			/* Strip valid characters from number */
			if (temp.indexOf('-') != -1)
				temp = (temp.split('-')).join('');
			if (temp.indexOf(' ') != -1)
				temp = (temp.split(' ')).join('');
			return ((temp.substring(0, 3) != '000') && (temp.substring(3, 5) != '00') && (temp.substring(5, 9) != '0000'));
		} catch (ex) {
			return (false);
		}
	},
	/**
	 * This method, isValidCreditCard, checks to see if the passed
	 * /p_cardNumberId/ is a valid credit card number based on the passed
	 * /p_cardTypeId/ and the Luhn Formula. The following credit cards may be
	 * tested with this method:
	 * 		- Visa has a length of 16 numbers and starts with 4 (dashes are
	 * 			optional)
	 * 		- MasterCard has a length of 16 numbers and starts with 51 through 55
	 * 			(dashes are optional)
	 * 		- Discover has a length of 16 numbers and starts with 6011 (dashes are
	 * 			optional)
	 * 		- American Express has a length of 15 numbers and starts with 34 or 37
	 * 		- Diners Club has a length of 14 numbers and starts with 30, 36, or 38
	 *
	 * @member Form.Validation
	 * @param {String} p_cardTypeId The name of the input field to get the card
	 * 		type from.
	 * @param {String} p_cardNumberId The name of the input field to get the card
	 * 		number from.
	 * @return Returns whether the card number is in the correct syntax and has a
	 * 		valid checksum.
	 * @type Boolean
	 */
	isValidCreditCard: function(p_cardTypeId, p_cardNumberId) {
		var regExp = '';
		var type = $F(p_cardTypeId);
		var number = $F(p_cardNumberId);

		/* Is the card type Visa? [length 16; prefix 4] */
		if (type == "Visa")
			regExp = /^4\d{3}-?\d{4}-?\d{4}-?\d{4}$/;
		/* Is the card type MasterCard? [length 16; prefix 51 - 55] */
		else if (type == "MasterCard")
			regExp = /^5[1-5]\d{2}-?\d{4}-?\d{4}-?\d{4}$/;
		/* Is the card type Discover? [length 16; prefix 6011] */
		else if (type == "Discover")
			regExp = /^6011-?\d{4}-?\d{4}-?\d{4}$/;
		/* Is the card type American Express? [length 15; prefix 34 or 37] */
		else if (type == "AmericanExpress")
			regExp = /^3[4,7]\d{13}$/;
		/* Is the card type Diners Club? [length 14; prefix 30, 36, or 38] */
		else if (type == "Diners")
			regExp = /^3[0,6,8]\d{12}$/;
		/* Does the card number have a valid syntax? */
		if (!regExp.test(number))
			return (false);

		/* Strip valid characters from number */
		number = (number.split('-')).join('');
		number = (number.split(' ')).join('');

		var checksum = 0;

		/* Luhn Formula */
		for (var i = (2 - (number.length % 2)), il = number.length; i <= il; i += 2)
			checksum += parseInt(number.charAt(i - 1));
		for (var i = (number.length % 2) + 1, il = number.length; i < il; i += 2) {
			var digit = parseInt(number.charAt(i - 1)) * 2;

			checksum += ((digit < 10) ? digit : (digit - 9));
		}
		return (!(checksum % 10) && checksum)
	}
};