/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/*
 * Example 15-5. Added functionality for Ajax in the Form.Validation object.
 */

/**
 * This method, ajaxCheck, provides on-the-fly validation on the client and
 * server by using Ajax to allow the server to validate the passed /p_element/'s
 * value based on the passed /p_validationType/ and optional /p_options/
 * parameters.
 *
 * @member Form.Validation
 * @param {Object} p_element The element to validate.
 * @param {String} p_validationType The type of validation the client and
 * 		server should provide.
 * @param {string} p_options Optional string to provide when validating credit
 * 		cards to provide the card type.
 * @return Returns true so that the blur will happen on the passed element as
 * 		it should.
 * @type Boolean
 * @see #isNumber
 * @see #isMoney
 * @see #isDate
 * @see #isPhoneNumber
 * @see #isEmail
 * @see #isSSN
 * @see #isValidCreditCard
 * @see #reportError
 * @see #reportSuccess
 * @see Ajax#Request
 */
ajaxCheck: function(p_element, p_validationType, p_options) {
	var validated = false;

	/* Is the validation type for validating a number? */
	if (p_validationType == 'number')
		validated = this.isNumber(p_element);
	/* Is the validation type for validating a monetary value? */
	else if (p_validationType == 'money')
		validated = this.isMoney(p_element);
	/* Is the validation type for validating a date? */
	else if (p_validationType == 'date')
		validated = this.isDate(p_element);
	/* Is the validation type for validating a phone number? */
	else if (p_validationType == 'phone')
		validation = this.isPhoneNumber(p_element);
	/* Is the validation type for validating an email address? */
	else if (p_validationType == 'isValidEmail')
		validation = this.isEmail(p_element);
	/* Is the validation type for validating a Social Security number? */
	else if (p_validationType == 'ssn')
		validation = this.isSSN(p_element);
	/* Is the validation type for validating a credit card? */
	else if (p_validationType == 'cc')
		validation = this.isValidCreditCard(p_options, p_element);
	/* Did client-side validation succeed? */
	if (validation) {
		new Ajax.Request('ajaxCheck.php', {
			method: 'get',
			parameters: {
			value: $F(p_element),
			type: p_validationType,
			options: p_options },
			onSuccess: function(xhrResponse) {
				/* Did the value not validate on the server? */
				if (xhrResponse.responseText == '0')
					this.reportError(p_element.id, p_validationType);
				else
					this.reportSuccess(p_element.id);
			}
		});
	} else
	this.reportError(p_element.id, p_validationType);
	return (true);
},
/**
 * This method, reportError, creates an element to put next to the form
 * field that did not validate correctly with a supplied message alerting the
 * user that there is a problem.
 *
 * @member Form.Validation
 * @param {String} p_element The element id to place the new element next to.
 * @param {String} p_validationType The type of validation the client and
 * server provided.
 * @see #ajaxCheck
 * @see Element#addClassName
 */
reportError: function(p_id, p_validationType) {
	var message = '';

	/* Is the validation type for validating a number? */
	if (p_validationType == 'number')
		message = 'This field expects a number. Example: 31';
	/* Is the validation type for validating a monetary value? */
	else if (p_validationType == 'money')
		message = 'This field expects a monetary value. Example: $31.00';
	/* Is the validation type for validating a date? */
	else if (p_validationType == 'date')
		message = 'This field expects a date. Example: 01/01/2007';
	/* Is the validation type for validating a phone number? */
	else if (p_validationType == 'phone')
		message = 'This field expects a phone number. Example (800) 555-5555';
	/* Is the validation type for validating an email address? */
	else if (p_validationType == 'isValidEmail')
		message = 'This field expects a valid email account. Example: anthony3@holdener.com';
	/* Is the validation type for validating a Social Security number? */
	else if (p_validationType == 'ssn')
		message = 'This field expects a valid Social Security number. Example: 234-56-7890';
	/* Is the validation type for validating a credit card? */
	else if (p_validationType == 'cc')
		message = 'This field expects a valid credit card number. Example: 4123 4567 8901 2349';
	/* There was an unknown validation type */
	else
		message = 'The input in this field is invalid.';

	var span = document.createElement('span');

	span.appendChild(document.createTextNode(message));
	span.id = 'span' + p_id;
	Element.addClassName(span, 'validationError');
	$(p_id).parentNode.appendChild(span);
},
/**
 * This method, reportSuccess, checks to see if the passed /p_id/ has a
 * sibling element. If it does and that element is a <span> element with a class
 * name of /validationError/ then remove it.
 *
 * @param {String} p_element The element id to check for another element next to it.
 * @see #ajaxCheck
 * @see Element#hasClassName
 */
reportSuccess: function(p_id) {
	var elem = $(p_id);

	/* Does the element have another element next to it? */
	if (elem.nextSibling)
		/*
		 * Is the other element a <span> element with a class name of
		 * /validationError/?
		 */
		if (elem.nextSibling.nodeName == 'SPAN' && Element.hasClassName(elem.nextSibling, 'validationError'))
		$(p_id).parentNode.removeChild(elem.nextSibling);
}