/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, character.js, encapsulates all of the logic and code
 * needed to create a character in the game.
 *
 * This code requires the Prototype library.
 */

/**
 * This class, character, will store all of the functionality needed by a
 * character in the game, including animation, movement, and statistics.
 */
var character = Class.create();
character.prototype = {
	/**
	 * This member, _handle, stores the <div> element (presumably) that stands in
	 * for the image.
	 * @private
	 */
	 _handle: null,
	/**
	 * This member, animation, holds all of the methods for creating the illusion
	 * of animation for the character and allows the control of that animation
	 * in the browser.
	 *
	 * @member character
	 */
	animation: {
		/**
		 * This member, _options, holds all of the developer-definable options
		 * for the animation portion of the character class.
		 * @private
		 */
		_options: {
			/**
			 * This member, frameSize, stores the width of an individual "frame"
			 * in the image.
			 */
			frameSize: 0,
			/**
			 * This member, frameCount, stores the number of frames contained in
			 * the image for a single sprite.
			 */
			frameCount: 0,
			/**
			 * This member, spriteSize, stores the height of an individual "sprite"
			 * in the image.
			 */
			spriteSize: 0,
			/**
			 * This member, spriteCount, stores the number of sprites contained in
			 * the image for the character.
			 */
			spriteCount: 0,
			/**
			 * This member, pauseTime, stores the length of time that the
			 * animation should pause between "frames" in milliseconds.
			 */
			pauseTime: 0
		},
		/**
		 * This member, _currentFrame, stores the "frame" currently being viewed
		 * in the browser.
		 * @private
		 */
		_currentFrame: 0,
		/**
		 * This member, _spriteCount, stores the "sprite" currently being viewed
		 * in the browser.
		 * @private
		 */
		_spriteFrame: 0,
		/**
		 * This member, _internalTimer, stores the switching time for the object.
		 * @private
		 */
		_internalTimer: null,
		/**
		 * This method, advanceFrame, changes the position of the background
		 * image of the /_handle/ based on the /_currentFrame/, /frameSize/,
		 * /spriteSize/, /_spriteCount/, and /_spriteFrame/.
		 *
		 * @member animation
		 */
		advanceFrame: function() {
			/* Has the animation reached the last image? */
			if (this._currentFrame == this._options.frameCount)
				this._currentFrame = 0;
			this._handle.setStyle({
				backgroundPosition: (-1 * this._options.frameSize * this._currentFrame) + 'px ' + (-1 * this._options.spriteSize * this._spriteFrame) + 'px'
			});
			this._currentFrame++;
		},
		/**
		 * This method, startAnimation, calls the DOM function /setInterval/ to
		 * start the timer for the animation and will report its success.
		 *
		 * @member animation
		 * @param {Integer} p_direction The offset for the necessary sprite of the character.
		 * @return Whether or not the animation was started.
		 * @type Boolean
		 */
		startAnimation: function(p_direction) {
			if (p_direction > this._options.spriteCount)
				return (false);
			this._spriteFrame = p_direction;
			this._internalTimer = setInterval(this.advanceFrame.bind(this), this._options.pauseTime);
			return (this._intervalTimer);
		},
		/**
		 * This method, pauseAnimation, calls the DOM function /clearInterval/
		 * to clear the timer for the animation and stop it in its current frame.
		 *
		 * @member animation
		 * @return Whether or not the animation was correctly paused.
		 * @type Boolean
		 */
		pauseAnimation: function() {
			clearInterval(this._internalTimer.bind(this));
			/* Has the timer been cleared? */
			if (!this._internalTimer) {
				this._internalTimer = null;
				return (true);
			}
			return (false);
		},
		/**
		 * This method, stopAnimation, calls the DOM function /clearInterval/
		 * to clear the timer for the animation, then the /_currentFrame/ is
		 * reset to 0 and the image reset to its first "frame".
		 *
		 * @member animation
		 * @return Whether or not the animation was correctly stopped.
		 * @type Boolean
		 */
		stopAnimation: function() {
			clearInterval(this._internalTimer.bind(this));
			/* Has the timer been cleared? */
			if (!this._internalTimer) {
				this._internalTimer = null;
				this._spriteFrame = 0;
				this._currentFrame = 0;
				this._handle.setStyle({
					backgroundPosition: '0 0'
				});
				return (true);
			}
			return (false);
		}
	},
	/**
	 * This method, initialize, is the constructor for the class and sets all
	 * of the necessary private members.
	 *
	 * @member character
	 * @constructor
	 * @param {String | Object} p_handle The id or object that represents the character.
	 * @param {Object} p_options The options to set for the animation.
	 */
	initialize: function(p_handle, p_options) {
		this._handle = $(p_handle);
		Object.extend(this.animation._options, p_options || {});
	},
	/**
	 * This method, startAnimation, is the public way of starting the animation
	 * of the character, by calling the /animation.startAnimation/ method.
	 *
	 * @member character
	 * @param {Integer} p_direction The offset for the necessary sprite of the character.
	 * @return Whether or not the character began being animated.
	 * @type Boolean
	 */
	startAnimation: function(p_direction) {
		return (this.animation.startAnimation(p_direction));
	},
	/**
	 * This method, pauseAnimation, is the public way of pausing the animation
	 * of the character, by calling the /animation.pauseAnimation/ method.
	 *
	 * @member character
	 * @return Whether or not the character paused its animation.
	 * @type Boolean
	 */
	pauseAnimation: function() {
		return (this.animation.pauseAnimation());
	},
	/**
	 * This method, stopAnimation, is the public way of stopping the animation
	 * of the character, by calling the /animation.stopAnimation/ method.
	 *
	 * @member character
	 * @return Whether or not the character stopped being animated.
	 * @type Boolean
	 */
	stopAnimation: function() {
		return (this.animation.stopAnimation());
	}
};