/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, character.js, encapsulates all of the logic and
 * code needed to create a character in the game.
 *
 * This code requires the Prototype library.
 */

/**
 * This variable, __PULSE_MOVEMENT__, is intended as a constant for movement timing.
 */
var __PULSE_MOVEMENT__ = 10;

/**
 * This variable, __MOVE_BACKWARD__, is intended as a constant for static movement
 * attached to the KEY_UP key.
 */
var __MOVE_BACKWARD__ = 0;
/**
 * This variable, __MOVE_RIGHT__, is intended as a constant for static movement
 * attached to the KEY_RIGHT key.
 */
var __MOVE_RIGHT__ = 1;
/**
 * This variable, __MOVE_FORWARD__, is intended as a constant for static movement
 * attached to the KEY_DOWN key.
 */
var __MOVE_FORWARD__ = 2;
/**
 * This variable, __MOVE_LEFT__, is intended as a constant for static movement
 * attached to the KEY_LEFT key.
 */
var __MOVE_LEFT__ = 3;

/**
 * This class, character, will store all of the functionality needed by a character
 * in the game, including animation, movement, and statistics.
 */
var character = Class.create();
character.prototype = {
	/**
	 * This member, _handle, stores the <div> element (presumably) that stands in
	 * for the image.
	 * @private
	 */
	_handle: null,
	/**
	 * This member, movement, holds all of the methods for making the character move
	 * around within the constraints of the game in the browser.
	 *
	 * @member character
	 */
	movement: {
		/**
		 * This member, _direction, holds the value of the current direction
		 * the character is traveling in.
		 * @private
		 */
		_direction: -1,
		/**
		 * This member, _moving, lets methods within the movement functionality
		 * know whether the character is currently moving or not.
		 * @private
		 */
		_moving: false,
		/**
		 * This member, _position, holds the current x, y coordinates of the
		 * character on the screen.
		 * @private
		 */
		_position: [0, 0],
		/**
		 * This member, _internalTimer, stores the movement time for the object.
		 * @private
		 */
		_internalTimer: null,
		/**
		 * This method, changeDirection, controls the movement of the character
		 * in any given direction, and controls starting and stopping the movement.
		 *
		 * @member movement
		 * @param {Integer} p_direction The direction the character should move in.
		 * @return Whether or not the character changed directions.
		 * @type Boolean
		 */
		changeDirection: function(p_direction) {
			/* Should the character be stopped? */
			if (this._moving && this._direction == p_direction) {
				this.stop();
				return (false);
			}
			this._direction = p_direction;
			/* Do we need to start the character? */
			if (!this._moving)
				this.start();
			return (true);
		},
		/**
		 * This method, start, starts the character moving in a given direction.
		 *
		 * @member movement
		 * @return Whether or not the character started moving.
		 * @type Boolean
		 */
		start: function( ) {
			this._internalTimer = setInterval(this.move.bind(this), __PULSE_MOVEMENT__);
			this._moving = true;
			return (this._intervalTimer);
		},
		/**
		 * This method, stop, stops the character's movement.
		 *
		 * @member movement
		 * @return Whether or not the character stopped moving.
		 * @type Boolean
		 */
		stop: function() {
			clearInterval(this._internalTimer.bind(this));
			/* Has the timer been cleared? */
			if (!this._internalTimer) {
				this._internalTimer = null;
				this._moving = false;
				return (true);
			}
			return (false);
		},
		/**
		 * This method, move, changes the coordinates of the character in the
		 * set direction.
		 *
		 * @member movement
		 */
		move: function() {
			switch (this._direction) {
				case __MOVE_BACKWARD__:
					this._position[1] -= 2;
					break;
				case __MOVE_RIGHT__:
					this._position[0] += 2;
					break;
				case __MOVE_FORWARD__:
					this._position[1] += 2;
					break;
				case __MOVE_LEFT__:
					this._position[0] -= 2;
					break;
			}
			this.position();
		},
		/**
		 * This method, position, does the actual moving of the character by
		 * changing the CSS to meet the new coordinates set.
		 *
		 * @member movement
		 */
		position: function() {
			this._handle.setStyle({
				left: this._position[0] + 'px',
				top: this._position[1] + 'px'
			});
		}
	},
	/**
	 * This method, moveCharacter, is the public method by which the character
	 * class should move an object in the game.
	 *
	 * @member character
	 * @param {Integer} p_direction The direction the character should move in.
	 */
	moveCharacter: function(p_direction) {
		try {
			this.movement.changeDirection(p_direction);
		} catch (ex) {
			alert('Illegal argument sent to method character.moveCharacter( ):', ex.description);
		}
	}
}