/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * This object, Logic, is the container for all mathematical logic functionality
 * for the game.
 */
var Logic = {
	/**
	 * This method, Bresenham, creates a list of points that draw an
	 * approximation of the mathematically correct line, which is described in
	 * the form of a linear function, as defined with the Bresenham line
	 * algorithm.
	 *
	 * @member Logic
	 * @param {Integer} p_x1 The starting x-coordinate for the line.
	 * @param {Integer} p_y1 The starting y-coordinate for the line.
	 * @param {Integer} p_x2 The ending x-coordinate for the line.
	 * @param {Integer} p_y2 The ending y-coordinate for the line.
	 * @return An array containing the list of points that creates the Bresenham line.
	 * @type Array
	 */
	Bresenham: function(p_x1, p_y1, p_x2, p_y2) {
		var deltaX = Math.abs(p_x2 - p_x1) << 1;
		var deltaY = Math.abs(p_y2 - p_y1) << 1;
		/*
		 * If p_x1 == p_x2 or p_y1 == p_y2, then it makes no difference what
		 * is set here
		 */
		var ix = ((p_x2 > p_x1) ? 1 : -1);
		var iy = ((p_y2 > p_y1) ? 1 : -1);
		var path = [];
		var i = 0;

		/* Which way is the line going to slope? */
		if (deltaX >= deltaY) {
			/* Error may go below zero */
			var error = deltaY - (deltaX >> 1);

			/* Loop until the points are connected */
			while (p_x1 != p_x2) {
				/* Is there a chance the line must shift? */
				if (error >= 0)
					/* Do we need to slide over? */
					if (error || (ix > 0)) {
						p_y1 += iy;
						error -= deltaX;
					}
				p_x1 += ix;
				error += deltaY;
				path[i++] = [p_x1, p_y1];
			}
		} else {
			/* Error may go below zero */
			var error = deltaX - (deltaY >> 1);

			/* Loop until the points are connected */
			while (p_y1 != p_y2) {
				/* Is there a chance the line must shift? */
				if (error >= 0)
					/* Do we need to slide over? */
					if (error || (iy > 0)) {
						p_x1 += ix;
						error -= deltaY;
					}
				p_y1 += iy;
				error += deltaX;
				path[i++] = [p_x1, p_y1];
			}
		}
		return (path);
	}
};