/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, character.js, encapsulates all of the logic and code
 * needed to create a character in the game.
 *
 * This code requires the Prototype library.
 */

/**
 * This class, character, will store all of the functionality needed by a
 * character in the game, including animation, movement, and statistics.
 */
var character = Class.create();
character.prototype = {
	/**
	 * This member, _handle, stores the <div> element (presumably) that stands in
	 * for the image.
	 * @private
	 */
	_handle: null,
	/**
	 * This member, movement, holds all of the methods for making the character
	 * move around within the constraints of the game in the browser.
	 *
	 * @member character
	 */
	movement: {
		/**
		 * This member, _moving, lets methods within the movement functionality
		 * know whether the character is currently moving or not.
		 * @private
		 */
		_moving: false,
		/**
		 * This member, _position, holds the current x, y coordinates of the
		 * character on the screen.
		 * @private
		 */
		_position: [0, 0],
		/**
		 * This member, _path, is a queue of points set by the Bresenham line
		 * algorithm between the current character position and the mouse
		 * event's coordinates.
		 * @private
		 */
		_path: [],
		/**
		 * This member, _internalTimer, stores the movement time for the object.
		 * @private
		 */
		_internalTimer: null,
		/**
		 * This method, changeDirectionMouse, controls the movement of the
		 * character for a mouse click, and controls starting and stopping the
		 * movement.
		 *
		 * @member movement
		 * @param {Integer} p_x The x-coordinate the character should move to.
		 * @param {Integer} p_y The y-coordinate the character should move to.
		 * @return Whether or not the character changed directions.
		 * @type Boolean
		 */
		changeDirectionMouse: function(p_x, p_y) {
			/* Should the character be stopped? */
			if (this._moving)
				this.stop();
			this._direction = -1;
			this._path = Logic.Bresenham(this._position[0], this._position[1], p_x, p_y);
			this.startMouse();
			return (true);
		},
		/**
		 * This method, startMouse, starts the character moving in a given direction.
		 *
		 * @member movement
		 * @return Whether or not the character started moving.
		 * @type Boolean
		 */
		startMouse: function() {
			this._internalTimer = setInterval(this.moveMouse.bind(this), __PULSE_MOVEMENT__);
			this._moving = true;
			return (this._intervalTimer);
		},
		/**
		 * This method, stop, stops the character's movement.
		 *
		 * @member movement
		 * @return Whether or not the character stopped moving.
		 * @type Boolean
		 */
		stop: function() {
			clearInterval(this._internalTimer.bind(this));
			/* Has the timer been cleared? */
			if (!this._internalTimer) {
				this._internalTimer = null;
				this._moving = false;
				return (true);
			}
			return (false);
		},
		/**
		 * This method, moveMouse, changes the coordinates of the character with
		 * the point in the path queue.
		 *
		 * @member movement
		 */
		moveMouse: function() {
			var point = this._path.shift();
			this._position[0] = point[0];
			this._position[1] = point[1];
			this.position( );
		},
		/**
		 * This method, position, does the actual moving of the character by
		 * changing the CSS to meet the new coordinates set.
		 *
		 * @member movement
		 */
		position: function() {
			this._handle.setStyle({
				left: this._position[0] + 'px',
				top: this._position[1] + 'px'
			});
		}
	},
	/**
	 * This method, moveCharacterMouse, is the public method by which the
	 * character class should move an object in the game when a mouse event
	 * happens.
	 *
	 * @member character
	 * @param {Integer} p_x The x-coordinate the character should move to.
	 * @param {Integer} p_y The y-coordinate the character should move to.
	 */
	moveCharacterMouse: function(p_x, p_y) {
		try {
			this.movement.changeDirectionMouse(p_x, p_y);
		} catch (ex) {
			alert('Illegal argument sent to method character.moveCharacterMouse():', ex.description);
		}
	}
}
