/* 
 * This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * This class, character, will store all of the functionality needed by a
 * character in the game, including animation, movement, and statistics.
 */
var character = Class.create();
character.prototype = {
	//.
	//.
	//.
	/**
	 * This member, collision, holds all of the methods for testing collisions
	 * between this character and other objects in the game.
	 *
	 * @member character
	 */
	collision: {
		/**
		 * This member, _boundings, holds the bounding box coordinates for
		 * the character.
		 * @private
		 */
		_boundings: [],
		/**
		 * This method, SetBounding, sets the new bounding box coordinates of
		 * the character with the passed points.
		 *
		 * @member collision
		 * @param {Array} p_points An array containing the new bounding box
		 *	 points of the character.
		 */
		SetBounding: function(p_points) {
			try {
				this._bounding[0] = [p_points[0][0], p_points[0][1]];
				this._bounding[1] = [p_points[1][0], p_points[1][1]];
			} catch (ex) {
				this._bounding = null;
			}
		},
		/**
		 *  This method, GetBoundX, gets one of the current x-coordinates of
		 *  the character.
		 *
		 * @member collision
		 * @param {Integer} p_corner The coordinate to return (1 or 2 for x1 and x2).
		 * @return The x-coordinate.
		 * @type Integer
		 */
		GetBoundX: function(p_corner) {
			/* Does the character have a bounding box? */
			if (this._bounding)
				return (this._bounding[p_corner - 1][0]);
			return (null);
		},
		/**
		 *  This method, GetBoundY, gets one of the current y-coordinates of
		 *  the character.
		 *
		 * @member collision
		 * @param {Integer} p_corner The coordinate to return (1 or 2 for y1 and y2).
		 * @return The y-coordinate.
		 * @type Integer
		 */
		GetBoundY: function(p_corner) {
			/* Does the character have a bounding box? */
			if (this._bounding)
				return (this._bounding[p_corner - 1][1]);
			return (null);
		},
		/**
		 * This method, GetBoundingBox, gets the current bounding box coordinates
		 * of the character.
		 *
		 * @member collision
		 * @return The current bounding box coordinates of the character.
		 * @type Array
		 */
		GetBoundingBox() {
			/* Does the character have a bounding box? */
			if (this._bounding)
				return ([[this._bounding[0]], this._bounding[1]]);
			return (null);
		}
	},
	/**
	 * This method, GetBoundX, is the public way of getting one of the current
	 * x-coordinates of the character.
	 *
	 * @member character
	 * @param {Integer} p_corner The coordinate to return (1 or 2 for x1 and x2).
	 * @return The x-coordinate.
	 * @type Integer
	 */
	GetBoundX: function(p_corner) {
		return (this.collision.GetBoundX(p_corner));
	},
	/**
	 * This method, GetBoundY, is the public way of getting one of the current
	 * y-coordinates of the character.
	 *
	 * @member character
	 * @param {Integer} p_corner The coordinate to return (1 or 2 for y1 and y2).
	 * @return The y-coordinate.
	 * @type Integer
	 */
	GetBoundY: function(p_corner) {
		return (this.collision.GetBoundY(p_corner));
	},
	/**
	 * This method, GetBoundingBox, is the public way of getting the current
	 * bounding box coordinates of the character.
	 *
	 * @member character
	 * @return The current bounding box coordinates of the character.
	 * @type Array
	 */
	GetBoundingBox: function() {
		return (this.collision.GetBoundingBox());
	},
	/**
	 * This method, TestCollision, tests for a collision between the character's
	 * bounding box and the box that was passed using rectangular collision
	 * detection.
	 *
	 * @member character
	 * @param {Array} p_box The bounding box to test against.
	 * @return Whether or not the passed bounding box collided with the
	 *	 character's bounding box.
	 * @type Boolean
	 */
	TestCollision: function(p_box) {
		return (Logic.RectangularCollisionDetect(this.GetBoundingBox(), p_box));
	}
}