/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview Example 7-6, filemenu.js: Code for manipulating the file menu
 *
 * This file, filemenu.js, contains the fileMenu object which is used to create
 * instances of a file menu on the page.
 */

/* Create a new class using Prototype's Class object */
var fileMenu = Class.create( );
/**
 * This object, fileMenu, creates the functionality for a file menu on the page.
 */
fileMenu.prototype = {
	/**
	 * This member, _menu, holds the id of this file menu.
	 * @private
	 */
	_menu: null,
	/**
	 * This member, _activeButton, holds the element that is currently active.
	 * @private
	 */
	_activeButton: null,
	/**
	 * This method, initialize, is the constructor for the class. Any members
	 * that need to be initialized should be here.
	 *
	 * @member fileMenu
	 * @constructor
	 * @param {String} p_element The element that represents the file menu.
	 */
	initialize: function(p_element) {
		/*
		 * Currently unused, but nice to have for multiple instances of
		 * the object
		 */
		this._menu = p_element;
	},
	/**
	 * This member, pageMousedown, is called on every mousedown event on the page
	 * and determines if the menu should be reset based on where the user clicks on
	 * the page.
	 *
	 * @member fileMenu
	 * @param {Object} e The event that called the method.
	 * @return Returns false so that no other event will be triggered.
	 * @type Boolean
	 * @see #getContainerWith
	 * @see #resetButton
	 */
	pageMousedown: function(e) {
		var target = null;

		/* Is the file menu active? */
		if (!this._activeButton)
			return;
		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			target = window.event.srcElement;
		else
			target = (e.target.tagName ? e.target : e.target.parentNode);
		/* Is the event target the active button? */
		if (this._activeButton == target)
			return;
		/* Is the target not part of the file menu? */
		if (!this.getContainerWith(target, 'UL', 'fileMenuChild')) {
			this.resetButton(this._activeButton);
			this._activeButton = null;
		}
		return (false);
	},
	/**
	 * This method, buttonClick, is called when the user clicks on one of the
	 * buttons that are on the main menu bar. It determines if where the user
	 * clicked on the menu bar is the active button, or if it is a different button
	 * and another button's drop-down menus may need to be cleaned up and reset.
	 *
	 * @member fileMenu
	 * @param {Object} e The event that called the method.
	 * @param {String} p_fileMenuId The id of the file menu that is being used.
	 * @return Returns false so that no other event will be triggered.
	 * @type Boolean
	 * @see #fileMenuInit
	 * @see #resetButton
	 * @see #depressButton
	 * @see #buttonMouseover
	 */
	buttonClick: function(e, p_fileMenuId) {
		var button = null;

		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			button = window.event.srcElement;
		else
			button = e.currentTarget;
		/* Blur the focus of the button here to remove the annoying outline */
		button.blur( );
		/* Is this button part of the file menu already? */
		if (!button.fileMenu) {
			button.fileMenu = $(p_fileMenuId);
			/* Is this button already initialized? */
			if (!button.fileMenu.isInitialized)
				this.fileMenuInit(button.fileMenu);
		}
		/* Is there an active button already? */
		if (this._activeButton)
			this.resetButton(this._activeButton);
		/* Is the button already activated? */
		if (button != this._activeButton) {
			this.depressButton(button);
			this._activeButton = button;
		} else
			this._activeButton = null;
		return (false);
	},
	/**
	 * This member, buttonMouseover, is called on a mouseover event on a button on
	 * the main menu bar of the file menu. If a different button was already active,
	 * then activate the current one instead.
	 *
	 * @member fileMenu
	 * @param {Object} e The event that called the method.
	 * @param {String} p_fileMenuId The id of the file menu that is being used.
	 * @see #buttonClick
	 */
	buttonMouseover: function(e, p_fileMenuId) {
		var button = null;
		
		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			button = window.event.srcElement;
		else
			button = e.currentTarget;
		/* Should this button be activated? */
		if (this._activeButton && this._activeButton != button)
			this.buttonClick(e, p_fileMenuId);
	},
	/**
	 * This method, depressButton, is called on a buttonClick when a new drop-down
	 * menu needs to be activated and positioned.
	 *
	 * @member fileMenu
	 * @param {Object} p_button The button that has been pressed.
	 * @see #getPageOffsetLeft
	 * @see #getPageOffsetTop
	 * @see Element#addClassName
	 * @see Element#setStyle
	 * @see #buttonClick
	 */
	depressButton: function(p_button) {
		var x, y;

		/*
		 * Make the button look depressed (no, not sad) and show the drop down
		 * associated with it
		 */
		$(p_button).addClassName('fileMenuButtonActive');
		/* Position any associated drop down under the button and display it */
		x = this.getPageOffsetLeft(p_button);
		y = this.getPageOffsetTop(p_button) + p_button.offsetHeight;
		/* Is the client Internet Explorer? */
		if (Browser.isIE) {
			x -= p_button.offsetWidth;
			y += p_button.offsetParent.clientTop;
		}
		$(p_button).setStyle({
			left: x + 'px',
			top: y + 'px',
			display: 'block'
		});
	},
	/**
	 * This method, resetButton, does what it says; it resets the button, closing
	 * all submenus.
	 *
	 * @member fileMenu
	 * @param {Object} p_button The button that has been pressed.
	 * @see #closeSubFileMenu
	 * @see Element#removeClassName
	 * @see Element#setStyle
	 * @see #pageMousedown
	 * @see #buttonClick
	 */
	resetButton: function(p_button) {
		$(p_button).removeClassName('fileMenuButtonActive');
		/* Does the button have a file menu? */
		if (p_button.fileMenu) {
			this.closeSubFileMenu(p_button.fileMenu);
			$(p_button).setStyle({ display: 'none' });
		}
	},
	/**
	 * This method, fileMenuMouseover, is called on a mouseover MouseEvent over any
	 * of the drop-down menus in the file menu bar. Its main purpose is to close
	 * submenus when they should no longer be active.
	 *
	 * @member fileMenu
	 * @param {Object} e The event that called the method.
	 * @see #getContainerWith
	 * @see #closeSubFileMenu
	 * @see Element#hasClassName
	 */
	fileMenuMouseover: function(e) {
		var fileMenu;

		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			fileMenu = this.getContainerWith(window.event.srcElement, 'UL', 'fileMenuChild');
		else
			fileMenu = e.currentTarget;
		/* Does this menu have submenus? */
		if (fileMenu.activeItem && ($(fileMenu.activeItem).hasClassName('fileMenuButton') && $(fileMenu.parentNode.firstChild).hasClassName('fileMenuItem')))
			this.closeSubFileMenu(fileMenu);
	},
	/**
	 * This method, fileMenuItemMouseover, is called when there is a mouseover event
	 * on one of the menu items that has a submenu attached to it. The method
	 * calculates the position where the submenu should be placed in relation to the
	 * menu item of the event.
	 *
	 * @member fileMenu
	 * @param {Object} e The event that called the method.
	 * @param {String} p_fileMenuId The id of the file menu that is being used.
	 * @see #getContainerWith
	 * @see #closeSubFileMenu
	 * @see #fileMenuInit
	 * @see #getPageOffsetLeft
	 * @see #getPageOffsetTop
	 * @see Element#hasClassName
	 * @see Element#addClassName
	 * @see Element#setStyle
	 */
	fileMenuItemMouseover: function(e, p_fileMenuId) {
		var item, fileMenu, x, y;

		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			item = this.getContainerWith(window.event.srcElement, 'A', 'fileMenuItem');
		else
			item = e.currentTarget;
		fileMenu = this.getContainerWith(item, 'UL', 'fileMenuChild');
		/* Does the file menu have an active item? */
		if (fileMenu.activeItem)
			this.closeSubFileMenu(p_fileMenuId);
		/* Is there a file menu id? */
		if (p_fileMenuId) {
			fileMenu.activeItem = item;
		/* Does the class name already exist? */
		if (!$(item).hasClassName('fileMenuItemHighlight'))
			$(item).addClassName('fileMenuItemHighlight');
		/* Has the sub file menu been attached already? */
		if (item.subFileMenu == null) {
			item.subFileMenu = $(p_fileMenuId);
			/* Has the sub file menu already been initialized? */
			if (!item.subFileMenu.isInitialized)
				this.fileMenuInit(item.subFileMenu);
		}

		/* Calculate the x and y positions where the submenu should be placed */
		x = this.getPageOffsetLeft(item) + item.offsetWidth;
		y = this.getPageOffsetTop(item);
		/* Is the client Opera? */
		if (Browser.isOpera) {
			x = item.offsetWidth;
			y = item.offsetTop;
		}
		/* Is the client Internet Explorer? */
		if (Browser.isIE) {
			x -= (this._activeButton.offsetWidth * 2);
			y -= this._activeButton.offsetHeight;
		}

		var maxX, maxY;

		/* Is the client Internet Explorer? */
		if (Browser.isIE) {
			maxX = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft) + (document.documentElement.clientWidth != 0 ? document.documentElement.clientWidth : document.body.clientWidth);
			maxY = Math.max(document.documentElement.scrollTop, document.body.scrollTop) + (document.documentElement.clientHeight != 0 ? document.documentElement.clientHeight : document.body.clientHeight);
		}
		/* Is the client Opera? */
		if (Browser.isOpera) {
			maxX = document.documentElement.scrollLeft + window.innerWidth;
			maxY = document.documentElement.scrollTop + window.innerHeight;
		}
		/* Is the client Mozilla? */
		if (Browser.isMoz) {
			maxX = window.scrollX + window.innerWidth;
			maxY = window.scrollY + window.innerHeight;
		}
		maxX -= item.subFileMenu.offsetWidth;
		maxY -= item.subFileMenu.offsetHeight;
		/* Is the x coordinate bigger than the maximum it can be? */
		if (x > maxX)
			x = Math.max(0, x - item.offsetWidth  item.subFileMenu.offsetWidth + (menu.offsetWidth  item.offsetWidth));
		y = Math.max(0, Math.min(y, maxY));
		/* Show the submenu */
		$(item).setStyle({
			left: x + 'px',
			top: y + 'px',
			display: 'block'
		});
		/* Is the client Internet Explorer? */
		if (Browser.isIE)
			window.event.cancelBubble = true;
		else
			e.stopPropagation();
		}
	},
	/**
	 * This method, closeSubFileMenu, hides the submenu from the user and then
	 * turns off all references to the submenu and that it was active.
	 *
	 * @member fileMenu
	 * @param {Object} p_fileMenu The file menu that is to be closed.
	 * @see #resetButton
	 * @see #fileMenuMouseover
	 * @see #fileMenuItemMouseover
	 */
	closeSubFileMenu: function(p_fileMenu) {
		/* Does the file menu not exist or is it not active? */
		if (!p_fileMenu || !p_fileMenu.activeItem)
			return;
		/* Does the file menu have an active sub file menu? */
		if (p_fileMenu.activeItem.subFileMenu) {
			this.closeSubFileMenu(p_fileMenu.activeItem.subFileMenu);
			$(p_fileMenu.activeItem.subFileMenu).setStyle({ display: none });
			p_fileMenu.activeItem.subFileMenu = null;
		}
		$(p_fileMenu.activeItem).removeClassName('fileMenuItemHighlight');
		p_fileMenu.activeItem = null;
	},
	/**
	 * This method, fileMenuInit, goes through a submenu and associates all of
	 * the children to the menu as a part of the menu. It also sets up the
	 * offset sizes of the submenu for positioning of the menu.
	 *
	 * @member fileMenu
	 * @param {Object} p_fileMenu The file menu that is to be closed.
	 * @see Element#setStyle
	 * @see Element#hasClassName
	 * @see #buttonClick
	 * @see #fileMenuItemMouseover
	 */
	fileMenuInit: function(p_fileMenu) {
		var itemList, spanList, textElement, arrowElement, itemWidth, w, dw;
		
		/* Is the client Internet Explorer? */
		if (Browser.isIE) {
			$(p_fileMenu).setStyle({ lineHeight: '2.5ex' });
			spanList = p_fileMenu.getElementsByTagName('span');
			/* Loop through the <span> elements */
			for (var i = 0, il = spanList.length; i < il; i++)
				/* Does the <span> element have the class name? */
				if ($(spanList[i]).hasClassName('fileMenuItemArrow')) {
					$(spanList[i]).setStyle({ fontFamily: 'Webdings' });
					spanList[i].firstChild.nodeValue = '4';
				}
		}
		itemList = p_fileMenu.getElementsByTagName('a');
		/* Does the itemList have any <a> elements? */
		if (itemList.length > 0)
			itemWidth = itemList[0].offsetWidth;
		else
			return;
		/* Loop through the <a> elements */
		for (var i = 0, il = itemList.length; i < il; i++) {
			spanList = itemList[i].getElementsByTagName('span');
			textElement = null;
			arrowElement = null;
			/* Loop through the <span> elements */
			for (var j = 0, jl = spanList.length; j < jl; j++) {
				/* Does the <span> element have the class name? */
				if ($(spanList[j]).hasClassName('fileMenuItemText'))
					textElement = spanList[j];
				/* Does the <span> element have the class name? */
				if ($(spanList[j]).hasClassName('fileMenuItemArrow'))
					arrowElement = spanList[j];
			}
		}
		/* Do the /textElement/ and /arrowElement/ exist? */
		if (textElement && arrowElement) {
			$(textElement).setStyle({
				paddingRight: (itemWidth - (textElement.offsetWidth + arrowElement.offsetWidth)) + 'px'
			});
			/* Is the client Opera? */
			if (Browser.isOpera)
				$(arrowElement).setStyle({ marginRight: '0' });
		}
		/* Is the client Internet Explorer? */
		if (Browser.isIE) {
			w = itemList[0].offsetWidth;
			$(itemList[0]).setStyle({ width: w + 'px' });
			dw = itemList[0].offsetWidth - w;
			w -= dw;
			$(itemList[0]).setStyle({ width: w + 'px' });
		}
		p_fileMenu.isInitialized = true;
	},
	/**
	 * This method, getContainerWith, finds the element with a given tag and
	 * class name and returns the discovered element or or null.
	 *
	 * @member fileMenu
	 * @param {Object} p_element The element to check.
	 * @param {String} p_tagname The tag name to look for.
	 * @param {String} p_className The class name to look for.
	 * @return Returns the discovered element, if found, or null.
	 * @type Object
	 * @see Element#hasClassName
	 * @see #pageMousedown
	 * @see #fileMenuMouseover
	 * @see #fileMenuItemMouseover
	 */
	getContainerWith: function(p_element, p_tagname, p_className) {
		/* Traverse the element tree while there are elements */
		while (p_element) {
			/* Does the element have the correct tag name and class name? */
			if (p_element.tagName && p_element.tagName == p_tagname && $(p_element).hasClassName(p_className))
				return (p_element);
			p_element = p_element.parentNode;
		}
		return (p_element);
	},
	/**
	 * This method, getPageOffsetLeft, returns the left offset of the element
	 * in relation to the page.
	 *
	 * @member fileMenu
	 * @param {Object} p_element The element to get the offset from.
	 * @return Returns the left page offset of the element.
	 * @type Integer
	 * @see #getPageOffsetLeft
	 * @see #depressButton
	 * @see #fileMenuItemMouseover
	 * @see #fileMenuItemMouseover
	 */
	getPageOffsetLeft: function(p_element) {
		var x;

		x = p_element.offsetLeft;
		/* Is the client not Mozilla and does the element have a parent offset? */
		if (!Browser.isMoz && p_element.offsetParent)
			x += this.getPageOffsetLeft(p_element.offsetParent);
		return (x);
	},
	/**
	 * This method, getPageOffsetTop, returns the top offset of the element in
	 * relation to the page.
	 *
	 * @member fileMenu
	 * @param {Object} p_element The element to get the offset from.
	 * @return Returns the left page offset of the element.
	 * @type Integer
	 * @see #getPageOffsetLeft
	 * @see #depressButton
	 * @see #fileMenuItemMouseover
	 * @see #fileMenuItemMouseover
	 */
	getPageOffsetTop: function(p_element) {
		var y;

		y = p_element.offsetTop;
		/* Is the client not Mozilla and does the element have a parent offset? */
		if (!Browser.isMoz && p_element.offsetParent)
			y += this.getPageOffsetTop(p_element.offsetParent);
		return (y);
	}
};

/* Create a new instance of the fileMenu class (this calls fileMenu.initialize( )) */
var menu = new fileMenu('fileMenu');

try {
	/*
	 * Set an event listener on the document for all mousedown events and
	 * have the system call the /pageMousedown( )/ method, binding it to the menu
	 * object that was just created. This allows for the creation of multiple
	 * file menus, if there is ever a need.
	 */
	Event.observe(document, 'mousedown', menu.pageMousedown.bind(menu), true);
} catch (ex) {}