/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview Example 7-7, tabs.js: The JavaScript for dynamic tab content.
 *
 * This file, tabs.js, contains the tabNavigation object which is used to create
 * instances of a tab navigation system on the page.
 */

/* Create a new class using Prototype's Class object */
var tabNavigation = Class.create( );
/**
 * This object, tabNavigation, provides the developer with the means of
 * creating a tabbed navigation system on the page.
 */
tabNavigation.prototype = {
	/**
	 * This member, _tabs, holds the id of the tabbed navigation.
	 * @private
	 */
	_tabs: null,
	/**
	 * This member, _previousTab, holds the id of the last tab clicked by the user.
	 * @private
	 */
	_previousTab: null,
	/**
	 * This member, initialize, is the constructor for the class. Any members that
	 * need to be initialized are done here, as well as any other initial
	 * housecleaning.
	 *
	 * @member tabNavigation
	 * @constructor
	 * @param {String} p_id The id of the element that represents the tabbed
	 * navigation.
	 * @param {String} p_startTab The id of the starting tab element.
	 * @see #expandTab
	 */
	initialize: function(p_id, p_startTab) {
		this._tabs = p_id;

		/* Get a list of link elements found in the tab list */
		var tabLinks = $(this._tabs).getElementsByTagName('a');

		/* Add a click event to all of the link elements */
		for (var i = tabLinks.length - 1; i >= 0;)
			tabLinks[i--].setAttribute('onclick', 'return tabs.expandTab(this.id);');
		/* Expand the starting tab */
		this.expandTab(p_startTab);
	},
	/**
	 * This method, expandTab, is called on all click MouseEvents associated with
	 * the tab navigation and hides the contents of the previous tab before showing
	 * the contents of the current tab.
	 *
	 * @member tabNavigation
	 * @param {String} p_linkId The id of the tab to expand.
	 * @return Returns false so that no other events fire after this one.
	 * @type Boolean
	 * @see #highlightTab
 	 * @see Element#setStyle
	 */
	expandTab: function(p_linkId) {
		var catId;

		this.highlightTab(p_linkId);
		/* Is there a previous tab selected */
		if (this._previousTab)
			$(this._previousTab).setStyle( { display: 'none' });
		catId = p_linkId + 'Content';
		$(catId).setStyle({ display: 'block' });
		this._previousTab = catId;
		return (false);
	},
	/**
	 * This member, highlightTab, is called from the expandTab method and removes the
	 * CSS rule for highlighting on all of the tabs and then sets it on the current
	 * tab.
	 *
	 * @member tabNavigation
	 * @param {String} p_linkId The id of the tab to expand.
	 */
	highlightTab: function(p_linkId) {
		var tabLinks = $(this._tabs).getElementsByTagName('a');

		/* Loop through the list of <a> elements */
		for (var i = tabLinks.length - 1; i >= 0;)
			$(tabLinks[i--]).removeClassName('selected');
		$(p_linkId).addClassName('selected');
	}
};

var tabs;

try {
	/*
	 * Set an event listener on the document for the load event and have the system
	 * create a new instance of the tabNavigation class (this calls
	 * tabNavigation.initialize( )). This allows for the creation of multiple file
	 * menus, if there is ever a need.
	 */
	Event.observe(window, 'load', function( ) {
		tabs = new tabNavigation('tabList', 'xhtml');
	}, true);
} catch(ex) {}