/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */
/**
 * @fileoverview This file, slideshow.js, builds up a list of images dynamically
 * through continuous /XMLHttpRequest/ calls to the server for data until all
 * pictures have been placed in the list. This allows the application to load
 * faster, as it does not have to load all of the images before the page is
 * functional. The application then allows users to view the list of images as
 * a slide show, viewing one image (and all of its associated data) at a time.
 *
 */

/**
 * This variable holds the current image number to be loaded.
 */
var imageNumber;
/**
 * This variable holds the extra image information (title, description).
 */
var imageData = [];
/**
 * This variable holds the index of which picture is being viewed.
 */
var index = 0;

/**
 * This function, setupApp, sets the initial image number to start pulling from
 * the server, then calls the /fetchNextImage/ function which starts the Ajax
 * calls.
 *
 * @see fetchNextImage
 */
function setupApp( ) {
	imageNumber = 0;
	fetchNextImage( );
}

/**
 * This function, fetchNextImage, checks to make sure it should make an Ajax call
 * based on the image number, and then calls the Prototype Framework's
 * /Ajax.Request/ method and creates a function to handle the results.
 *
 * @see Ajax#Request
 */
function fetchNextImage( ) {
	/* Is the image number bigger than 0? */
	if (++imageNumber > 0) {
		/* Call sendPhoto.php with the number of the photo */
		new Ajax.Request('sendPhoto.php', {
			method: 'post',
			parameters: 'number=' + imageNumber,
			/*
			 * The onSuccess method checks to see if the number of elements sent
			 * via XML is greater than one (one means there was an error). If
			 * it is, then it creates a new list item and image, placing the
			 * latter inside the former before adding the Base64encoded string
			 * into the image's src.
			 */
			onSuccess: function(xhrResponse) {
				/* Did we get an XML response we want? */
				if (xhrResponse .responseXML.documentElement.childNodes.length > 1) {
					/* Create new elements within the DOM document */
					var newItem = document.createElement('li');
					var newImage = document.createElement('img');

					/*
					 * Add id attributes to both and put the image in the list
					 * item
					 */
					newImage.setAttribute('id', 'i' + imageNumber);
					newItem.appendChild(newImage);
					newItem.setAttribute('id', 'l' + imageNumber);

					/* Add the new image to the list, then hide it */
					$('slideshowList').appendChild(newItem);
					Element.hide($('l' + imageNumber));

					/* Add the Base64-encoded string */
					$('i' + imageNumber).src = 'data:image/jpg;base64,' + xhrResponse.responseXML.documentElement.getElementsByTagName('image')[0].firstChild.nodeValue;
					/* Create the next index in the array to hold the image data */
					imageData[(imageNumber - 1)] = [];
					imageData[(imageNumber - 1)][0] = xhrResponse.responseXML.documentElement.getElementsByTagName('title')[0].firstChild.nodeValue;
					imageData[(imageNumber - 1)][1] = xhrResponse.responseXML.documentElement.getElementsByTagName('description')[0].firstChild.nodeValue;

					/* Is this the first image? */
					if (imageNumber <= 1) {
						/* Set the initial image and show it */
						index = 0;
						Effect.Appear($('l' + imageNumber));
						$('imageTitle').innerHTML = imageData[0][0];
						$('imageDescription').innerHTML = imageData[0][1];
					}
					/* Recursive call! */
					fetchNextImage( );
				} else {
					/* We are done */
					imageNumber = -1;
				}
			}
		});
	}
}

/**
 * This function, changeSlide, moves the display of individual <li> elements one
 * element at a time while hiding all other elements to give the illusion of moving
 * back and forth through a slide show of <li> elements.
 *
 * @param {Integer} p_slideDirection The direction of the slide change (-1 back
 * 		and 1 forward).
 * @return Returns false so that the element that had the event click stops any
 * 		default events.
 * @type Boolean
 * @see Element#hide
 * @see Effect#Appear
 */
function changeSlide(p_slideDirection) {
	/* Is the index going to be too small or too large? */
	if (!((index + p_slideDirection) < 0 || (index + p_slideDirection) > $('slideshowList').childNodes.length - 1)) {
		index += p_slideDirection;

		/* Loop through the unordered list and hide all images */
		var items = $('slideshowList').getElementsByTagName('li');

		for (var i = 0, il = items.length; i < il; i++)
			Element.hide(items[i]);
		/*
		 * Now make the image to be changed appear, and change the title and
		 * description.
		 */
		Effect.Appear($('slideshowList').childNodes[index]);
		$('imageTitle').innerHTML = imageData[index][0];
		$('imageDescription').innerHTML = imageData[index][1];
	}
	/* Return false so that the links do not try to actually go somewhere */
	return (false);
}

try {
	/* Call /setupApp( )/ when the page is loaded */
	Event.observe(window, 'load', setupApp, false);
} catch (ex) {}