/* This example is from the book _Ajax: The Definitive Guide_ by Anthony T. Holdener III.
 * Written by Anthony T. Holdener III.  Copyright (C) 2008 O'Reilly Media, Inc.
 * You may study, use, modify, and distribute this example for any purpose.
 * This example is provided WITHOUT WARRANTY either expressed or implied.
 */

/* This code is necessary for browsers that do not define DOM constants */
if (document.ELEMENT_NODE == null) {
	document.ELEMENT_NODE = 1;
	document.ATTRIBUTE_NODE = 2;
	document.TEXT_NODE = 3;
	document.CDATA_SECTION_NODE = 4;
	document.ENTITY_REFERENCE_NODE = 5;
	document.ENTITY_NODE = 6;
	document.PROCESSING_INSTRUCTION_NODE = 7;
	document.COMMENT_NODE = 8;
	document.DOCUMENT_NODE = 9;
	document.DOCUMENT_TYPE_NODE = 10;
	document.DOCUMENT_FRAGMENT_NODE = 11;
	document.NOTATION_NODE = 12;
}

/**
 * This class, tableSort, is created as a class and not just an object so that the
 * page may have more than one table with the ability to sort, without having to
 * go to a lot of trouble in code to keep them separate within the logic itself.
 *
 * @requires Class#create
 * @see Class#create
 */
var tableSort = Class.create( );
tableSort.prototype = {
	/**
	 * /multipleWS/ will hold the regular expression to eliminate multiple whitespace
	 * in a string.
	 * @private
	 */
	multipleWS: '',
	/**
	 * /endWS/ will hold the regular expresssion to eliminate whitespace at the end
	 * of the string.
	 * @private
	 */
	endWS: '',
	/**
	 * /tblElement/ will hold the table's <body> element that is to be sorted.
	 * @private
	 */
	tblElement: null,
	/**
	 * /lastColumn/ will hold the last column sorted by the user.
	 * @private
	 */
	lastColumn: null,
	/**
	 * /reverseSort/ will contain an array of columns of the table and will keep
	 * track of the sort direction the table should take.
	 * @private
	 */
	reverseSort: new Array( ),
	/**
	 * This method, initialize, is the constructor for the /tableSort/ class. It
	 * seeds values into the private members of the class based on the passed
	 * <table> element /p_id/ variable and constant regular expressions.
	 *
	 * @param {String} p_id The id attribute of the table to sort.
	 * @constructor
	 */
	initialize: function(p_id) {
		this.tblElement = $(p_id).getElementsByTagName('tbody')[0];
		this.endWS = new RegExp("^\\s*|\\s*$", "g");
		this.multipleWS = new RegExp("\\s\\s+", "g");
	},
	/* Normalize the data contained in the element into a single string */
	/**
	 * This method, normalizeElement, takes the passed /p_element/ and strips out
	 * all element tags, etc. from the node and any /childNodes/ the element may
	 * contain, returning a string with only the text data that was held in the
	 * element.
	 *
	 * @param {Node} p_element The element that is to be normalized.
	 * @return Returns the normalized string without element tags or extra
	 * whitespace.
	 * @type String
	 * @private
	 * @see #sortColumn
	 */
	normalizeElement: function(p_element) {
		/* The variable to hold the normalized string */
		var normalize = '';

		/*
		 * Loop through the passed element's /childNodes/ to normalize them
		 * as well.
		 */
		for (var i = 0, il = p_element.childNodes.length; i < il;) {
			/* The child element to check */
			var el = p_element.childNodes[i++];

			/* Is this node a text node, CDATA node, or comment node? */
			if (el.nodeType == document.TEXT_NODE || el.nodeType == document.CDATA_SECTION_NODE || el.nodeType == document.COMMENT_NODE)
				normalize += el.nodeValue;
			/* Is this node an element node and a <br> element? */
			else if (el.nodeType = document.ELEMENT_NODE && el.tagName == 'BR')
				normalize += ' ';
			/* This is something to normalize */
			else
				normalize += this.normalizeElement(el);
		}
		return (this.stripSpaces(normalize));
	},
	/* Strip any unnecessary whitespace from the string */
	/**
	 * This method, stripSpaces, takes the passed /p_string/ variable and using
	 * regular expressions, strips all unnecessary whitespace from the string
	 * before returning it.
	 *
	 * @param {String} p_string The string to be stripped of whitespace.
	 * @return Returns the passed string stripped of unnecessary whitespace.
	 * @type String
	 * @private
	 * @see #normalizeElement
	 */
	stripSpaces: function(p_string) {
		p_string = p_string.replace(this.multipleWS, ' ');
		p_string = p_string.replace(this.endWS, '');
		return (p_string);
	},
	/**
	 * This method, compareNodes, takes two node values as parameters (/p1/ and
	 * /p2/) and compares them to each other. It sends back a value based on
	 * the following:
	 *
	 * 1 - /p1/ is greater than /p2/
	 * 0 - /p1/ is equal to /p2/
	 * -1 - /p1/ is less than /p2/
	 *
	 * @param {String} p1 The first node value in the test.
	 * @param {String} p2 The second node value in the test.
	 * @return Returns a value based on the rules in the description of this method.
	 * @type Integer
	 * @private
	 * @see #sortColumn
	 */
	compareNodes: function(p1, p2) {
		/* Convert the values, if possible, to Floats */
		var f1 = parseFloat(p1), f2 = parseFloat(p2);

		/* Are both values numbers? (they are faster to sort) */
		if (!isNaN(f1) && !isNaN(f2)) {
			p1 = f1;
			p2 = f2;
		}
		/* Are the two values the same? */
		if (p1 == p2)
			return 0;
		/* Is the first value larger than the second value? */
		if (p1 > p2)
			return 1;
		return -1;
	},
	/**
	 * This method, sortColumn, sorts the passed /p_column/ in a direction based on
	 * the passed /p_defaultDirection/, moving data in all sibling columns when it
	 * sorts.
	 *
	 * @param {Integer} p_column The column index that is to be sorted.
	 * @param {Integer} p_defaultDirection The direction the sort should take (1 for
	 * !default).
	 * @see #normalizeElement
	 * @see #compareNodes
	 */
	sortColumn: function(p_column, p_defaultDirection) {
		var tempDisplay = this.tblElement.style.display;
		var j, index;

		/* Set a default direction if one is not passed in */
		if (defaultDirection == null)
			defaultDirection = 0;
		/*
		* Has the passed column been sorted yet? - if not, set its initial sorting
		* direction.
		*/
		if (this.reverseSort[p_column] == null)
			this.reverseSort[p_column] = p_defaultDirection;
		/*
		* Was the lastColumn sorted the passed column? - if it is, reverse the sort
		* direction.
		*/
		if (this.lastColumn == p_column)
			this.reverseSort[p_column] = !this.reverseSort[p_column];
		this.lastColumn = p_column;
		/*
		* Hide the table during the sort to avoid flickering in the table and
		* misrendering issues found in Netscape 6
		*/
		this.tblElement.style.display = 'none';
		/* Loop through each row of the table and compare it to other row values */
		for (var i = 0, il = this.tblElement.rows.length; i < il; i++) {
			index = this.normalizeElement(this.tblElement.rows[i].cells[p_column]);
			j = i;
			/*
			* Sort through all of the previous records and insert any of these rows
			* in front of the current row if such action is warranted.
			*/
			while (j > 0) {
				var doSort = this.compareNodes(this.normalizeElement(this.tblElement.rows[j - 1].cells[p_column]), index);

				/*
				* Doing the opposite sort direction for alternating clicks is as
				* simple as negating the current value for the sort order,
				* turning Ascending to Descending and back again.
				*/
				if (this.reverseSort[p_column])
					doSort = -doSort;
				if (doSort > 0)
					this.tblElement.insertBefore(this.tblElement.rows[j],vthis.tblElement.rows[j - 1]);
				j -= 1;
			}
		}
		/* Set the table's display to what it was before the sort */
		this.tblElement.style.display = tempDisplay;
		/* Do not let the onclick event know that it worked */
		return (false);
	}
};